///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

/**
 * \file     fmpq_poly-test.c
 * \brief    Test file for code coverage in fmpq_poly.c and fmpq_poly.h
 * \author   Sebastian Pancratz
 * \date     March 2010 -- July 2010
 * \version  0.1.7
 * 
 * The code in this file aims to cover the code provided by the rational 
 * function field implementation in the file <tt>fmpq_poly.c</tt> and 
 * <tt>fmpq_poly.h</tt>.
 * 
 * For almost all functions that are available through the header file 
 * <tt>fmpq_poly.h</tt>, there is a corresponding test function in this 
 * file.
 * 
 * Typically, for a function taking \f$k\f$ arguments the testing method has 
 * the signature <tt>int test_foo(char * in1, ..., char * in<i>k</i>, 
 * char * out, int verbose)</tt>.  The function then sets up \f$k\f$ 
 * independent rational functions, computes the output and obtains a string 
 * representation, finally comparing this to the rational function represented 
 * by <tt>out</tt>.  The function returns <tt>0</tt> for a test passed and 
 * <tt>-1</tt> otherwise.  This idea is modified accordingly when the function 
 * which is to be tested takes further arguments of primitive type.  In 
 * addition to this, for some functions that have to be careful about handling 
 * input aliasing correctly, e.g. addition and multiplication, there are 
 * multiple testing functions simulating this.
 */

#ifdef __cplusplus
    extern "C" {
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "fmpq_poly.h"

int test_canonicalize(char * innum, fmpz_t inden, char * out, int verbose)
{
    int ans;
    fmpq_poly_t op;
    char * res;
    
    fmpq_poly_init(op);
    fmpz_poly_from_string(fmpq_poly_numref(op), innum);
    op->den = fmpz_realloc(op->den, fmpz_size(inden));
    fmpz_set(op->den, inden);
    fmpq_poly_canonicalize(op, NULL);
    
    res = fmpq_poly_to_string(op);
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_canonicalize: passed\n");
        }
        else
        {
            printf("test_numref: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

int test_numref(char * in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    
    res = fmpz_poly_to_string(fmpq_poly_numref(rop));
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_numref: passed\n");
        }
        else
        {
            printf("test_numref: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_denref(char * in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    mpz_t den;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    mpz_init(den);
    if (rop->den == NULL)
        mpz_set_si(den, 1);
    else
        fmpz_to_mpz(den, fmpq_poly_denref(rop));
    
    res = mpz_get_str(NULL, 10, den);
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_denref: passed\n");
        }
        else
        {
            printf("test_denref: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    mpz_clear(den);
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_set(char * in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(op);
    fmpq_poly_init(rop);
    fmpq_poly_from_string(op, in);
    fmpq_poly_set(rop, op);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_set: passed\n");
        }
        else
        {
            printf("test_set: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(op);
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_set_si(long in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_set_si(rop, in);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_set_si: passed\n");
        }
        else
        {
            printf("test_set_si: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_set_fmpz(fmpz_t in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_set_fmpz(rop, in);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_set_fmpz: passed\n");
        }
        else
        {
            printf("test_set_fmpz: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_set_mpz(mpz_t in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_set_mpz(rop, in);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_set_mpz: passed\n");
        }
        else
        {
            printf("test_set_mpz: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_swap(char * in1, char * in2, char * out1, char * out2, int verbose)
{
    int ans;
    fmpq_poly_t op1, op2;
    char * res1;
    char * res2;
    
    fmpq_poly_init(op1);
    fmpq_poly_init(op2);
    fmpq_poly_from_string(op1, in1);
    fmpq_poly_from_string(op2, in2);
    fmpq_poly_swap(op1, op2);
    
    res1 = fmpq_poly_to_string(op1);
    res2 = fmpq_poly_to_string(op2);
    ans = !strcmp(out1, res1) & !strcmp(out2, res2);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_swap: passed\n");
        }
        else
        {
            printf("test_swap: failed\n");
            printf("    Expected \"%s\" and \"%s\", got \"%s\" and \"%s\"\n", 
                out1, out2, res1, res2);
        }
    }
    
    fmpq_poly_clear(op1);
    fmpq_poly_clear(op2);
    free(res1);
    free(res2);
    
    return ans ? 1 : 0;
}

int test_zero(char * in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_zero(rop);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_zero: passed\n");
        }
        else
        {
            printf("test_zero: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_one(char * in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_one(rop);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_one: passed\n");
        }
        else
        {
            printf("test_one: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = -op
int test_neg1(char * in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_neg(rop, op);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_neg1: passed\n");
        }
        else
        {
            printf("test_neg1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = -rop
int test_neg2(char * in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_neg(rop, rop);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_neg2: passed\n");
        }
        else
        {
            printf("test_neg2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = 1/op
int test_inv1(char * in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_inv(rop, op);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_inv1: passed\n");
        }
        else
        {
            printf("test_inv1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = 1/rop
int test_inv2(char * in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_inv(rop, rop);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_inv2: passed\n");
        }
        else
        {
            printf("test_inv2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_get_coeff_mpq(char * in, long i, char * out, int verbose)
{
    int ans;
    fmpq_poly_t op;
    mpq_t coeff;
    char * res;
    
    mpq_init(coeff);
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_get_coeff_mpq(coeff, op, i);
    
    res = mpq_get_str(NULL, 10, coeff);
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_get_coeff_mpq: passed\n");
        }
        else
        {
            printf("test_get_coeff_mpq: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    mpq_clear(coeff);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

int test_set_coeff_fmpz(char * in, unsigned long i, fmpz_t x, char * out, int verbose)
{
    int ans;
    fmpq_poly_t op;
    char * res;
    
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_set_coeff_fmpz(op, i, x);
    
    res = fmpq_poly_to_string(op);
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_set_coeff_fmpz: passed\n");
        }
        else
        {
            printf("test_get_coeff_fmpz: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

int test_set_coeff_mpz(char * in, unsigned long i, mpz_t x, char * out, int verbose)
{
    int ans;
    fmpq_poly_t op;
    char * res;
    
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_set_coeff_mpz(op, i, x);
    
    res = fmpq_poly_to_string(op);
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_set_coeff_mpz: passed\n");
        }
        else
        {
            printf("test_get_coeff_mpz: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

int test_set_coeff_si(char * in, unsigned long i, long x, char * out, int verbose)
{
    int ans;
    fmpq_poly_t op;
    char * res;
    
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_set_coeff_si(op, i, x);
    
    res = fmpq_poly_to_string(op);
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_set_coeff_si: passed\n");
        }
        else
        {
            printf("test_get_coeff_si: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

int test_set_coeff_mpq(char * in, unsigned long i, mpq_t x, char * out, int verbose)
{
    int ans;
    fmpq_poly_t op;
    char * res;
    
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_set_coeff_mpq(op, i, x);
    
    res = fmpq_poly_to_string(op);
    ans = !strcmp(out, res);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_set_coeff_mpq: passed\n");
        }
        else
        {
            printf("test_set_coeff_mpq: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

int test_equal(char * in1, char * in2, int out, int verbose)
{
    int ans;
    fmpq_poly_t op1, op2;
    
    fmpq_poly_init(op1);
    fmpq_poly_init(op2);
    fmpq_poly_from_string(op1, in1);
    fmpq_poly_from_string(op2, in2);
    
    ans = fmpq_poly_equal(op1, op2) == out;
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_equal: passed\n");
        }
        else
        {
            printf("test_equal: failed\n");
            printf("    Expected \"%d\", got \"%d\"\n", out, fmpq_poly_equal(op1, op2));
        }
    }
    
    fmpq_poly_clear(op1);
    fmpq_poly_clear(op2);
    
    return ans ? 1 : 0;
}

int test_cmp(char * in1, char * in2, int out, int verbose)
{
    int ans;
    fmpq_poly_t op1, op2;
    
    fmpq_poly_init(op1);
    fmpq_poly_init(op2);
    fmpq_poly_from_string(op1, in1);
    fmpq_poly_from_string(op2, in2);
    
    ans = fmpq_poly_cmp(op1, op2) == out;
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_cmp: passed\n");
        }
        else
        {
            printf("test_cmp: failed\n");
            printf("    Expected \"%d\", got \"%d\"\n", out, fmpq_poly_cmp(op1, op2));
        }
    }
    
    fmpq_poly_clear(op1);
    fmpq_poly_clear(op2);
    
    return ans ? 1 : 0;
}

int test_length(char * in, unsigned long out, int verbose)
{
    int ans;
    fmpq_poly_t op;
    
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    
    ans = fmpq_poly_length(op) == out;
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_length: passed\n");
        }
        else
        {
            printf("test_length: failed\n");
            printf("    Expected \"%lu\", got \"%lu\"\n", out, fmpq_poly_length(op));
        }
    }
    
    fmpq_poly_clear(op);
    
    return ans ? 1 : 0;
}

int test_degree(char * in, long out, int verbose)
{
    int ans;
    fmpq_poly_t op;
    
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    
    ans = fmpq_poly_degree(op) == out;
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_length: passed\n");
        }
        else
        {
            printf("test_length: failed\n");
            printf("    Expected \"%li\", got \"%li\"\n", out, fmpq_poly_degree(op));
        }
    }
    
    fmpq_poly_clear(op);
    
    return ans ? 1 : 0;
}

enum BINOP { add, sub, mul, addmul, submul, floordiv, mod, gcd, lcm, compose };

//! rop = op1 * op2
int test_binop1(enum BINOP op, char * in1, char * in2, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op1, op2;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op1);
    fmpq_poly_init(op2);
    fmpq_poly_from_string(op1, in1);
    fmpq_poly_from_string(op2, in2);
    
    switch (op)
    {
        case add:       fmpq_poly_add(rop, op1, op2);      break;
        case sub:       fmpq_poly_sub(rop, op1, op2);      break;
        case mul:       fmpq_poly_mul(rop, op1, op2);      break;
        case addmul:    fmpq_poly_addmul(rop, op1, op2);   break;
        case submul:    fmpq_poly_submul(rop, op1, op2);   break;
        case floordiv:  fmpq_poly_floordiv(rop, op1, op2); break;
        case mod:       fmpq_poly_mod(rop, op1, op2);      break;
        case gcd:       fmpq_poly_gcd(rop, op1, op2);      break;
        case lcm:       fmpq_poly_lcm(rop, op1, op2);      break;
        case compose:   fmpq_poly_compose(rop, op1, op2);  break;
        default:
            printf("ERROR (test_binop1).  Unknown operation.\n");
            abort();
    }
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_binop1(%d): passed\n", (int)op);
        }
        else
        {
            printf("test_binop1(%d): failed\n", (int)op);
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op1);
    fmpq_poly_clear(op2);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = op1 * op1
int test_binop2(enum BINOP op, char * in1, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op1;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op1);
    fmpq_poly_from_string(op1, in1);
    
    switch (op)
    {
        case add:       fmpq_poly_add(rop, op1, op1);      break;
        case sub:       fmpq_poly_sub(rop, op1, op1);      break;
        case mul:       fmpq_poly_mul(rop, op1, op1);      break;
        case addmul:    fmpq_poly_addmul(rop, op1, op1);   break;
        case submul:    fmpq_poly_submul(rop, op1, op1);   break;
        case floordiv:  fmpq_poly_floordiv(rop, op1, op1); break;
        case mod:       fmpq_poly_mod(rop, op1, op1);      break;
        case gcd:       fmpq_poly_gcd(rop, op1, op1);      break;
        case lcm:       fmpq_poly_lcm(rop, op1, op1);      break;
        case compose:   fmpq_poly_compose(rop, op1, op1);  break;
        default:
            printf("ERROR (test_binop1).  Unknown operation.\n");
            abort();
    }
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_binop2(%d): passed\n", (int)op);
        }
        else
        {
            printf("test_binop2(%d): failed\n", (int)op);
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op1);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = rop * op2
int test_binop3(enum BINOP op, char * in1, char * in2, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op2;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op2);
    fmpq_poly_from_string(rop, in1);
    fmpq_poly_from_string(op2, in2);
    
    switch (op)
    {
        case add:       fmpq_poly_add(rop, rop, op2);      break;
        case sub:       fmpq_poly_sub(rop, rop, op2);      break;
        case mul:       fmpq_poly_mul(rop, rop, op2);      break;
        case addmul:    fmpq_poly_addmul(rop, rop, op2);   break;
        case submul:    fmpq_poly_submul(rop, rop, op2);   break;
        case floordiv:  fmpq_poly_floordiv(rop, rop, op2); break;
        case mod:       fmpq_poly_mod(rop, rop, op2);      break;
        case gcd:       fmpq_poly_gcd(rop, rop, op2);      break;
        case lcm:       fmpq_poly_lcm(rop, rop, op2);      break;
        case compose:   fmpq_poly_compose(rop, rop, op2);  break;
        default:
            printf("ERROR (test_binop3).  Unknown operation.\n");
            abort();
    }
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_binop3(%d): passed\n", (int)op);
        }
        else
        {
            printf("test_binop3(%d): failed\n", (int)op);
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op2);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = op1 * rop
int test_binop4(enum BINOP op, char * in1, char * in2, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op1;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op1);
    fmpq_poly_from_string(op1, in1);
    fmpq_poly_from_string(rop, in2);
    
    switch (op)
    {
        case add:       fmpq_poly_add(rop, op1, rop);      break;
        case sub:       fmpq_poly_sub(rop, op1, rop);      break;
        case mul:       fmpq_poly_mul(rop, op1, rop);      break;
        case addmul:    fmpq_poly_addmul(rop, op1, rop);   break;
        case submul:    fmpq_poly_submul(rop, op1, rop);   break;
        case floordiv:  fmpq_poly_floordiv(rop, op1, rop); break;
        case mod:       fmpq_poly_mod(rop, op1, rop);      break;
        case gcd:       fmpq_poly_gcd(rop, op1, rop);      break;
        case lcm:       fmpq_poly_lcm(rop, op1, rop);      break;
        case compose:   fmpq_poly_compose(rop, op1, rop);  break;
        default:
            printf("ERROR (test_binop4).  Unknown operation.\n");
            abort();
    }
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_binop4(%d): passed\n", (int)op);
        }
        else
        {
            printf("test_binop4(%d): failed\n", (int)op);
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op1);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = rop * rop
int test_binop5(enum BINOP op, char * in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    
    switch (op)
    {
        case add:       fmpq_poly_add(rop, rop, rop);      break;
        case sub:       fmpq_poly_sub(rop, rop, rop);      break;
        case mul:       fmpq_poly_mul(rop, rop, rop);      break;
        case addmul:    fmpq_poly_addmul(rop, rop, rop);   break;
        case submul:    fmpq_poly_submul(rop, rop, rop);   break;
        case floordiv:  fmpq_poly_floordiv(rop, rop, rop); break;
        case mod:       fmpq_poly_mod(rop, rop, rop);      break;
        case gcd:       fmpq_poly_gcd(rop, rop, rop);      break;
        case lcm:       fmpq_poly_lcm(rop, rop, rop);      break;
        case compose:   fmpq_poly_compose(rop, rop, rop);  break;
        default:
            printf("ERROR (test_binop5).  Unknown operation.\n");
            abort();
    }
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_binop5(%d): passed\n", (int)op);
        }
        else
        {
            printf("test_binop5(%d): failed\n", (int)op);
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = op1 * op2, with rop pre-set to inrop
int test_binop6(enum BINOP op, char * inrop, char * in1, char * in2, 
    char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op1, op2;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op1);
    fmpq_poly_init(op2);
    fmpq_poly_from_string(rop, inrop);
    fmpq_poly_from_string(op1, in1);
    fmpq_poly_from_string(op2, in2);
    
    switch (op)
    {
        case add:       fmpq_poly_add(rop, op1, op2);      break;
        case sub:       fmpq_poly_sub(rop, op1, op2);      break;
        case mul:       fmpq_poly_mul(rop, op1, op2);      break;
        case addmul:    fmpq_poly_addmul(rop, op1, op2);   break;
        case submul:    fmpq_poly_submul(rop, op1, op2);   break;
        case floordiv:  fmpq_poly_floordiv(rop, op1, op2); break;
        case mod:       fmpq_poly_mod(rop, op1, op2);      break;
        case gcd:       fmpq_poly_gcd(rop, op1, op2);      break;
        case lcm:       fmpq_poly_lcm(rop, op1, op2);      break;
        case compose:   fmpq_poly_compose(rop, op1, op2);  break;
        default:
            printf("ERROR (test_binop1).  Unknown operation.\n");
            abort();
    }
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_binop6(%d): passed\n", (int)op);
        }
        else
        {
            printf("test_binop6(%d): failed\n", (int)op);
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op1);
    fmpq_poly_clear(op2);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = x * op
int test_scalar_mul_si1(char * in, long x, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_scalar_mul_si(rop, op, x);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_scalar_mul_si1: passed\n");
        }
        else
        {
            printf("test_scalar_mul_si1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = x * rop
int test_scalar_mul_si2(char * in, long x, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_scalar_mul_si(rop, rop, x);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_scalar_mul_si2: passed\n");
        }
        else
        {
            printf("test_scalar_mul_si2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = x * op
int test_scalar_mul_mpz1(char * in, mpz_t x, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_scalar_mul_mpz(rop, op, x);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_scalar_mul_mpz1: passed\n");
        }
        else
        {
            printf("test_scalar_mul_mpz1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = x * rop
int test_scalar_mul_mpz2(char * in, mpz_t x, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_scalar_mul_mpz(rop, rop, x);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_scalar_mul_mpz2: passed\n");
        }
        else
        {
            printf("test_scalar_mul_mpz2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = x * op
int test_scalar_mul_mpq1(char * in, mpq_t x, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_scalar_mul_mpq(rop, op, x);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_scalar_mul_mpq1: passed\n");
        }
        else
        {
            printf("test_scalar_mul_mpq1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = x * rop
int test_scalar_mul_mpq2(char * in, mpq_t x, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_scalar_mul_mpq(rop, rop, x);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_scalar_mul_mpq2: passed\n");
        }
        else
        {
            printf("test_scalar_mul_mpq2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = op / x
int test_scalar_div_si1(char * in, long x, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_scalar_div_si(rop, op, x);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_scalar_div_si1: passed\n");
        }
        else
        {
            printf("test_scalar_div_si1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = rop / x
int test_scalar_div_si2(char * in, long x, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_scalar_div_si(rop, rop, x);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_scalar_div_si2: passed\n");
        }
        else
        {
            printf("test_scalar_div_si2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = op / x
int test_scalar_div_mpz1(char * in, mpz_t x, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_scalar_div_mpz(rop, op, x);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_scalar_div_mpz1: passed\n");
        }
        else
        {
            printf("test_scalar_div_mpz1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = rop / x
int test_scalar_div_mpz2(char * in, mpz_t x, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_scalar_div_mpz(rop, rop, x);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_scalar_div_mpz2: passed\n");
        }
        else
        {
            printf("test_scalar_div_mpz2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = op / x
int test_scalar_div_mpq1(char * in, mpq_t x, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_scalar_div_mpq(rop, op, x);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_scalar_div_mpq1: passed\n");
        }
        else
        {
            printf("test_scalar_div_mpq1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = rop / x
int test_scalar_div_mpq2(char * in, mpq_t x, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_scalar_div_mpq(rop, rop, x);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_scalar_div_mpq2: passed\n");
        }
        else
        {
            printf("test_scalar_div_mpq2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop1, rop2 = divrem(op1, op2)
int test_divrem1(char * in1, char * in2, char * out1, char * out2, int verbose)
{
    int ans;
    fmpq_poly_t rop1, rop2, op1, op2;
    char * res1;
    char * res2;
    
    fmpq_poly_init(rop1);
    fmpq_poly_init(rop2);
    fmpq_poly_init(op1);
    fmpq_poly_init(op2);
    fmpq_poly_from_string(op1, in1);
    fmpq_poly_from_string(op2, in2);
    fmpq_poly_divrem(rop1, rop2, op1, op2);
    
    res1 = fmpq_poly_to_string(rop1);
    res2 = fmpq_poly_to_string(rop2);
    ans = !strcmp(res1, out1) & !strcmp(res2, out2);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_divrem1: passed\n");
        }
        else
        {
            printf("test_divrem1: failed\n");
            printf("    Expected \"(%s, %s)\", got \"(%s, %s)\"\n", out1, out2, res1, res2);
        }
    }
    
    fmpq_poly_clear(rop1);
    fmpq_poly_clear(rop2);
    fmpq_poly_clear(op1);
    fmpq_poly_clear(op2);
    free(res1);
    free(res2);
    
    return ans ? 1 : 0;
}

//! rop1, rop2 = divrem(op1, op2)
int test_divrem2(char * in1, char * in2, char * out1, char * out2, int verbose)
{
    int ans;
    fmpq_poly_t rop1, rop2, op1, op2;
    char * res1;
    char * res2;
    
    fmpq_poly_init(rop1);
    fmpq_poly_init(rop2);
    fmpq_poly_init(op2);
    fmpq_poly_from_string(rop1, in1);
    fmpq_poly_from_string(op2, in2);
    fmpq_poly_divrem(rop1, rop2, rop1, op2);
    
    res1 = fmpq_poly_to_string(rop1);
    res2 = fmpq_poly_to_string(rop2);
    ans = !strcmp(res1, out1) & !strcmp(res2, out2);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_divrem2: passed\n");
        }
        else
        {
            printf("test_divrem2: failed\n");
            printf("    Expected \"(%s, %s)\", got \"(%s, %s)\"\n", out1, out2, res1, res2);
        }
    }
    
    fmpq_poly_clear(rop1);
    fmpq_poly_clear(rop2);
    fmpq_poly_clear(op2);
    free(res1);
    free(res2);
    
    return ans ? 1 : 0;
}

//! rop1, rop2 = divrem(op1, rop1)
int test_divrem3(char * in1, char * in2, char * out1, char * out2, int verbose)
{
    int ans;
    fmpq_poly_t rop1, rop2, op1;
    char * res1;
    char * res2;
    
    fmpq_poly_init(rop1);
    fmpq_poly_init(rop2);
    fmpq_poly_init(op1);
    fmpq_poly_from_string(op1, in1);
    fmpq_poly_from_string(rop1, in2);
    fmpq_poly_divrem(rop1, rop2, op1, rop1);
    
    res1 = fmpq_poly_to_string(rop1);
    res2 = fmpq_poly_to_string(rop2);
    ans = !strcmp(res1, out1) & !strcmp(res2, out2);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_divrem3: passed\n");
        }
        else
        {
            printf("test_divrem3: failed\n");
            printf("    Expected \"(%s, %s)\", got \"(%s, %s)\"\n", out1, out2, res1, res2);
        }
    }
    
    fmpq_poly_clear(rop1);
    fmpq_poly_clear(rop2);
    fmpq_poly_clear(op1);
    free(res1);
    free(res2);
    
    return ans ? 1 : 0;
}

//! rop1, rop2 = divrem(rop2, op2)
int test_divrem4(char * in1, char * in2, char * out1, char * out2, int verbose)
{
    int ans;
    fmpq_poly_t rop1, rop2, op2;
    char * res1;
    char * res2;
    
    fmpq_poly_init(rop1);
    fmpq_poly_init(rop2);
    fmpq_poly_init(op2);
    fmpq_poly_from_string(rop2, in1);
    fmpq_poly_from_string(op2, in2);
    fmpq_poly_divrem(rop1, rop2, rop2, op2);
    
    res1 = fmpq_poly_to_string(rop1);
    res2 = fmpq_poly_to_string(rop2);
    ans = !strcmp(res1, out1) & !strcmp(res2, out2);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_divrem4: passed\n");
        }
        else
        {
            printf("test_divrem4: failed\n");
            printf("    Expected \"(%s, %s)\", got \"(%s, %s)\"\n", out1, out2, res1, res2);
        }
    }
    
    fmpq_poly_clear(rop1);
    fmpq_poly_clear(rop2);
    fmpq_poly_clear(op2);
    free(res1);
    free(res2);
    
    return ans ? 1 : 0;
}

//! rop1, rop2 = divrem(op1, rop2)
int test_divrem5(char * in1, char * in2, char * out1, char * out2, int verbose)
{
    int ans;
    fmpq_poly_t rop1, rop2, op1;
    char * res1;
    char * res2;
    
    fmpq_poly_init(rop1);
    fmpq_poly_init(rop2);
    fmpq_poly_init(op1);
    fmpq_poly_from_string(op1, in1);
    fmpq_poly_from_string(rop2, in2);
    fmpq_poly_divrem(rop1, rop2, op1, rop2);
    
    res1 = fmpq_poly_to_string(rop1);
    res2 = fmpq_poly_to_string(rop2);
    ans = !strcmp(res1, out1) & !strcmp(res2, out2);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_divrem5: passed\n");
        }
        else
        {
            printf("test_divrem5: failed\n");
            printf("    Expected \"(%s, %s)\", got \"(%s, %s)\"\n", out1, out2, res1, res2);
        }
    }
    
    fmpq_poly_clear(rop1);
    fmpq_poly_clear(rop2);
    fmpq_poly_clear(op1);
    free(res1);
    free(res2);
    
    return ans ? 1 : 0;
}

//! rop1, rop2 = divrem(rop1, rop1)
int test_divrem6(char * in, char * out1, char * out2, int verbose)
{
    int ans;
    fmpq_poly_t rop1, rop2;
    char * res1;
    char * res2;
    
    fmpq_poly_init(rop1);
    fmpq_poly_init(rop2);
    fmpq_poly_from_string(rop1, in);
    fmpq_poly_divrem(rop1, rop2, rop1, rop1);
    
    res1 = fmpq_poly_to_string(rop1);
    res2 = fmpq_poly_to_string(rop2);
    ans = !strcmp(res1, out1) & !strcmp(res2, out2);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_divrem6: passed\n");
        }
        else
        {
            printf("test_divrem6: failed\n");
            printf("    Expected \"(%s, %s)\", got \"(%s, %s)\"\n", out1, out2, res1, res2);
        }
    }
    
    fmpq_poly_clear(rop1);
    fmpq_poly_clear(rop2);
    free(res1);
    free(res2);
    
    return ans ? 1 : 0;
}

//! rop1, rop2 = divrem(rop2, rop2)
int test_divrem7(char * in, char * out1, char * out2, int verbose)
{
    int ans;
    fmpq_poly_t rop1, rop2;
    char * res1;
    char * res2;
    
    fmpq_poly_init(rop1);
    fmpq_poly_init(rop2);
    fmpq_poly_from_string(rop2, in);
    fmpq_poly_divrem(rop1, rop2, rop2, rop2);
    
    res1 = fmpq_poly_to_string(rop1);
    res2 = fmpq_poly_to_string(rop2);
    ans = !strcmp(res1, out1) & !strcmp(res2, out2);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_divrem7: passed\n");
        }
        else
        {
            printf("test_divrem7: failed\n");
            printf("    Expected \"(%s, %s)\", got \"(%s, %s)\"\n", out1, out2, res1, res2);
        }
    }
    
    fmpq_poly_clear(rop1);
    fmpq_poly_clear(rop2);
    free(res1);
    free(res2);
    
    return ans ? 1 : 0;
}

//! rop = op**e
int test_power1(char * in, unsigned long exp, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_power(rop, op, exp);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_pow1: passed\n");
        }
        else
        {
            printf("test_pow1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = rop**e
int test_power2(char * in, unsigned long exp, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_power(rop, rop, exp);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_pow2: passed\n");
        }
        else
        {
            printf("test_pow2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

//! ropd, rops, ropt = xgcd(op1, op2)
int test_xgcd1(char * in1, char * in2, char * outd, char * outs, char * outt, int verbose)
{
    int ans;
    fmpq_poly_t ropd, rops, ropt, op1, op2;
    char * resd;
    char * ress;
    char * rest;
    
    fmpq_poly_init(ropd);
    fmpq_poly_init(rops);
    fmpq_poly_init(ropt);
    fmpq_poly_init(op1);
    fmpq_poly_init(op2);
    fmpq_poly_from_string(op1, in1);
    fmpq_poly_from_string(op2, in2);
    fmpq_poly_xgcd(ropd, rops, ropt, op1, op2);
    
    resd = fmpq_poly_to_string(ropd);
    ress = fmpq_poly_to_string(rops);
    rest = fmpq_poly_to_string(ropt);
    ans = !strcmp(resd, outd) & !strcmp(ress, outs) & !strcmp(rest, outt);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_xgcd1: passed\n");
        }
        else
        {
            printf("test_xgcd1: failed\n");
            printf("    Expected \"(%s, %s, %s)\", got \"(%s, %s, %s)\"\n", outd, outs, outt, resd, ress, rest);
        }
    }
    
    fmpq_poly_clear(ropd);
    fmpq_poly_clear(rops);
    fmpq_poly_clear(ropt);
    fmpq_poly_clear(op1);
    fmpq_poly_clear(op2);
    free(resd);
    free(ress);
    free(rest);
    
    return ans ? 1 : 0;
}

//! ropd, rops, ropt = xgcd(ropd, op2)
int test_xgcd2(char * in1, char * in2, char * outd, char * outs, char * outt, int verbose)
{
    int ans;
    fmpq_poly_t ropd, rops, ropt, op2;
    char * resd;
    char * ress;
    char * rest;
    
    fmpq_poly_init(ropd);
    fmpq_poly_init(rops);
    fmpq_poly_init(ropt);
    fmpq_poly_init(op2);
    fmpq_poly_from_string(ropd, in1);
    fmpq_poly_from_string(op2, in2);
    fmpq_poly_xgcd(ropd, rops, ropt, ropd, op2);
    
    resd = fmpq_poly_to_string(ropd);
    ress = fmpq_poly_to_string(rops);
    rest = fmpq_poly_to_string(ropt);
    ans = !strcmp(resd, outd) & !strcmp(ress, outs) & !strcmp(rest, outt);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_xgcd2: passed\n");
        }
        else
        {
            printf("test_xgcd2: failed\n");
            printf("    Expected \"(%s, %s, %s)\", got \"(%s, %s, %s)\"\n", outd, outs, outt, resd, ress, rest);
        }
    }
    
    fmpq_poly_clear(ropd);
    fmpq_poly_clear(rops);
    fmpq_poly_clear(ropt);
    fmpq_poly_clear(op2);
    free(resd);
    free(ress);
    free(rest);
    
    return ans ? 1 : 0;
}

//! ropd, rops, ropt = xgcd(rops, op2)
int test_xgcd3(char * in1, char * in2, char * outd, char * outs, char * outt, int verbose)
{
    int ans;
    fmpq_poly_t ropd, rops, ropt, op2;
    char * resd;
    char * ress;
    char * rest;
    
    fmpq_poly_init(ropd);
    fmpq_poly_init(rops);
    fmpq_poly_init(ropt);
    fmpq_poly_init(op2);
    fmpq_poly_from_string(rops, in1);
    fmpq_poly_from_string(op2, in2);
    fmpq_poly_xgcd(ropd, rops, ropt, rops, op2);
    
    resd = fmpq_poly_to_string(ropd);
    ress = fmpq_poly_to_string(rops);
    rest = fmpq_poly_to_string(ropt);
    ans = !strcmp(resd, outd) & !strcmp(ress, outs) & !strcmp(rest, outt);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_xgcd3: passed\n");
        }
        else
        {
            printf("test_xgcd3: failed\n");
            printf("    Expected \"(%s, %s, %s)\", got \"(%s, %s, %s)\"\n", outd, outs, outt, resd, ress, rest);
        }
    }
    
    fmpq_poly_clear(ropd);
    fmpq_poly_clear(rops);
    fmpq_poly_clear(ropt);
    fmpq_poly_clear(op2);
    free(resd);
    free(ress);
    free(rest);
    
    return ans ? 1 : 0;
}

//! ropd, rops, ropt = xgcd(ropt, op2)
int test_xgcd4(char * in1, char * in2, char * outd, char * outs, char * outt, int verbose)
{
    int ans;
    fmpq_poly_t ropd, rops, ropt, op1;
    char * resd;
    char * ress;
    char * rest;
    
    fmpq_poly_init(ropd);
    fmpq_poly_init(rops);
    fmpq_poly_init(ropt);
    fmpq_poly_init(op1);
    fmpq_poly_from_string(op1, in1);
    fmpq_poly_from_string(ropt, in2);
    fmpq_poly_xgcd(ropd, rops, ropt, op1, ropt);
    
    resd = fmpq_poly_to_string(ropd);
    ress = fmpq_poly_to_string(rops);
    rest = fmpq_poly_to_string(ropt);
    ans = !strcmp(resd, outd) & !strcmp(ress, outs) & !strcmp(rest, outt);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_xgcd4: passed\n");
        }
        else
        {
            printf("test_xgcd4: failed\n");
            printf("    Expected \"(%s, %s, %s)\", got \"(%s, %s, %s)\"\n", outd, outs, outt, resd, ress, rest);
        }
    }
    
    fmpq_poly_clear(ropd);
    fmpq_poly_clear(rops);
    fmpq_poly_clear(ropt);
    fmpq_poly_clear(op1);
    free(resd);
    free(ress);
    free(rest);
    
    return ans ? 1 : 0;
}

//! ropd, rops, ropt = xgcd(op1, ropd)
int test_xgcd5(char * in1, char * in2, char * outd, char * outs, char * outt, int verbose)
{
    int ans;
    fmpq_poly_t ropd, rops, ropt, op1;
    char * resd;
    char * ress;
    char * rest;
    
    fmpq_poly_init(ropd);
    fmpq_poly_init(rops);
    fmpq_poly_init(ropt);
    fmpq_poly_init(op1);
    fmpq_poly_from_string(op1, in1);
    fmpq_poly_from_string(ropd, in2);
    fmpq_poly_xgcd(ropd, rops, ropt, op1, ropd);
    
    resd = fmpq_poly_to_string(ropd);
    ress = fmpq_poly_to_string(rops);
    rest = fmpq_poly_to_string(ropt);
    ans = !strcmp(resd, outd) & !strcmp(ress, outs) & !strcmp(rest, outt);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_xgcd5: passed\n");
        }
        else
        {
            printf("test_xgcd5: failed\n");
            printf("    Expected \"(%s, %s, %s)\", got \"(%s, %s, %s)\"\n", outd, outs, outt, resd, ress, rest);
        }
    }
    
    fmpq_poly_clear(ropd);
    fmpq_poly_clear(rops);
    fmpq_poly_clear(ropt);
    fmpq_poly_clear(op1);
    free(resd);
    free(ress);
    free(rest);
    
    return ans ? 1 : 0;
}

//! ropd, rops, ropt = xgcd(op1, rops)
int test_xgcd6(char * in1, char * in2, char * outd, char * outs, char * outt, int verbose)
{
    int ans;
    fmpq_poly_t ropd, rops, ropt, op1;
    char * resd;
    char * ress;
    char * rest;
    
    fmpq_poly_init(ropd);
    fmpq_poly_init(rops);
    fmpq_poly_init(ropt);
    fmpq_poly_init(op1);
    fmpq_poly_from_string(op1, in1);
    fmpq_poly_from_string(rops, in2);
    fmpq_poly_xgcd(ropd, rops, ropt, op1, rops);
    
    resd = fmpq_poly_to_string(ropd);
    ress = fmpq_poly_to_string(rops);
    rest = fmpq_poly_to_string(ropt);
    ans = !strcmp(resd, outd) & !strcmp(ress, outs) & !strcmp(rest, outt);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_xgcd6: passed\n");
        }
        else
        {
            printf("test_xgcd6: failed\n");
            printf("    Expected \"(%s, %s, %s)\", got \"(%s, %s, %s)\"\n", outd, outs, outt, resd, ress, rest);
        }
    }
    
    fmpq_poly_clear(ropd);
    fmpq_poly_clear(rops);
    fmpq_poly_clear(ropt);
    fmpq_poly_clear(op1);
    free(resd);
    free(ress);
    free(rest);
    
    return ans ? 1 : 0;
}

//! ropd, rops, ropt = xgcd(op1, ropt)
int test_xgcd7(char * in1, char * in2, char * outd, char * outs, char * outt, int verbose)
{
    int ans;
    fmpq_poly_t ropd, rops, ropt, op1;
    char * resd;
    char * ress;
    char * rest;
    
    fmpq_poly_init(ropd);
    fmpq_poly_init(rops);
    fmpq_poly_init(ropt);
    fmpq_poly_init(op1);
    fmpq_poly_from_string(op1, in1);
    fmpq_poly_from_string(ropt, in2);
    fmpq_poly_xgcd(ropd, rops, ropt, op1, ropt);
    
    resd = fmpq_poly_to_string(ropd);
    ress = fmpq_poly_to_string(rops);
    rest = fmpq_poly_to_string(ropt);
    ans = !strcmp(resd, outd) & !strcmp(ress, outs) & !strcmp(rest, outt);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_xgcd7: passed\n");
        }
        else
        {
            printf("test_xgcd7: failed\n");
            printf("    Expected \"(%s, %s, %s)\", got \"(%s, %s, %s)\"\n", outd, outs, outt, resd, ress, rest);
        }
    }
    
    fmpq_poly_clear(ropd);
    fmpq_poly_clear(rops);
    fmpq_poly_clear(ropt);
    fmpq_poly_clear(op1);
    free(resd);
    free(ress);
    free(rest);
    
    return ans ? 1 : 0;
}

//! ropd, rops, ropt = xgcd(rops, ropt)
int test_xgcd8(char * in1, char * in2, char * outd, char * outs, char * outt, int verbose)
{
    int ans;
    fmpq_poly_t ropd, rops, ropt;
    char * resd;
    char * ress;
    char * rest;
    
    fmpq_poly_init(ropd);
    fmpq_poly_init(rops);
    fmpq_poly_init(ropt);
    fmpq_poly_from_string(rops, in1);
    fmpq_poly_from_string(ropt, in2);
    fmpq_poly_xgcd(ropd, rops, ropt, rops, ropt);
    
    resd = fmpq_poly_to_string(ropd);
    ress = fmpq_poly_to_string(rops);
    rest = fmpq_poly_to_string(ropt);
    ans = !strcmp(resd, outd) & !strcmp(ress, outs) & !strcmp(rest, outt);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_xgcd8: passed\n");
        }
        else
        {
            printf("test_xgcd8: failed\n");
            printf("    Expected \"(%s, %s, %s)\", got \"(%s, %s, %s)\"\n", outd, outs, outt, resd, ress, rest);
        }
    }
    
    fmpq_poly_clear(ropd);
    fmpq_poly_clear(rops);
    fmpq_poly_clear(ropt);
    free(resd);
    free(ress);
    free(rest);
    
    return ans ? 1 : 0;
}

//! ropd, rops, ropt = xgcd(ropd, rops)
int test_xgcd9(char * in1, char * in2, char * outd, char * outs, char * outt, int verbose)
{
    int ans;
    fmpq_poly_t ropd, rops, ropt;
    char * resd;
    char * ress;
    char * rest;
    
    fmpq_poly_init(ropd);
    fmpq_poly_init(rops);
    fmpq_poly_init(ropt);
    fmpq_poly_from_string(ropd, in1);
    fmpq_poly_from_string(rops, in2);
    fmpq_poly_xgcd(ropd, rops, ropt, ropd, rops);
    
    resd = fmpq_poly_to_string(ropd);
    ress = fmpq_poly_to_string(rops);
    rest = fmpq_poly_to_string(ropt);
    ans = !strcmp(resd, outd) & !strcmp(ress, outs) & !strcmp(rest, outt);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_xgcd9: passed\n");
        }
        else
        {
            printf("test_xgcd9: failed\n");
            printf("    Expected \"(%s, %s, %s)\", got \"(%s, %s, %s)\"\n", outd, outs, outt, resd, ress, rest);
        }
    }
    
    fmpq_poly_clear(ropd);
    fmpq_poly_clear(rops);
    fmpq_poly_clear(ropt);
    free(resd);
    free(ress);
    free(rest);
    
    return ans ? 1 : 0;
}

//! ropd, rops, ropt = xgcd(ropd, ropt)
int test_xgcd10(char * in1, char * in2, char * outd, char * outs, char * outt, int verbose)
{
    int ans;
    fmpq_poly_t ropd, rops, ropt;
    char * resd;
    char * ress;
    char * rest;
    
    fmpq_poly_init(ropd);
    fmpq_poly_init(rops);
    fmpq_poly_init(ropt);
    fmpq_poly_from_string(ropd, in1);
    fmpq_poly_from_string(ropt, in2);
    fmpq_poly_xgcd(ropd, rops, ropt, ropd, ropt);
    
    resd = fmpq_poly_to_string(ropd);
    ress = fmpq_poly_to_string(rops);
    rest = fmpq_poly_to_string(ropt);
    ans = !strcmp(resd, outd) & !strcmp(ress, outs) & !strcmp(rest, outt);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_xgcd10: passed\n");
        }
        else
        {
            printf("test_xgcd10: failed\n");
            printf("    Expected \"(%s, %s, %s)\", got \"(%s, %s, %s)\"\n", outd, outs, outt, resd, ress, rest);
        }
    }
    
    fmpq_poly_clear(ropd);
    fmpq_poly_clear(rops);
    fmpq_poly_clear(ropt);
    free(resd);
    free(ress);
    free(rest);
    
    return ans ? 1 : 0;
}

//! rop = diff(op)
int test_derivative1(char * in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_derivative(rop, op);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_derivative1: passed\n");
        }
        else
        {
            printf("test_derivative1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = diff(rop)
int test_derivative2(char * in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_derivative(rop, rop);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_derivative2: passed\n");
        }
        else
        {
            printf("test_derivative2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_evaluate_mpz(char * in, mpz_t a, char * out, int verbose)
{
    int ans;
    mpq_t rop;
    fmpq_poly_t op;
    char * res;
    
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    mpq_init(rop);
    fmpq_poly_evaluate_mpz(rop, op, a);
    
    res = mpq_get_str(NULL, 10, rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_evaluate_mpz: passed\n");
        }
        else
        {
            printf("test_evaluate_mpz: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(op);
    mpq_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_evaluate_mpq1(char * in, mpq_t a, char * out, int verbose)
{
    int ans;
    mpq_t rop;
    fmpq_poly_t op;
    char * res;
    
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    mpq_init(rop);
    fmpq_poly_evaluate_mpq(rop, op, a);
    
    res = mpq_get_str(NULL, 10, rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_evaluate_mpq1: passed\n");
        }
        else
        {
            printf("test_evaluate_mpq1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(op);
    mpq_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_evaluate_mpq2(char * in, mpq_t a, char * out, int verbose)
{
    int ans;
    mpq_t rop;
    fmpq_poly_t op;
    char * res;
    
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    mpq_init(rop);
    mpq_set(rop, a);
    fmpq_poly_evaluate_mpq(rop, op, rop);
    
    res = mpq_get_str(NULL, 10, rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_evaluate_mpq2: passed\n");
        }
        else
        {
            printf("test_evaluate_mpq2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(op);
    mpq_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_content(char * in, char * out, int verbose)
{
    int ans;
    mpq_t rop;
    fmpq_poly_t op;
    char * res;
    
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    mpq_init(rop);
    fmpq_poly_content(rop, op);
    
    res = mpq_get_str(NULL, 10, rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_content: passed\n");
        }
        else
        {
            printf("test_content: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(op);
    mpq_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = primitive_part(op)
int test_primitive_part1(char * in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_primitive_part(rop, op);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_primitive_part1: passed\n");
        }
        else
        {
            printf("test_primitive_part1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

//! rop = primitive_part(rop)
int test_primitive_part2(char * in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_primitive_part(rop, rop);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_primitive_part2: passed\n");
        }
        else
        {
            printf("test_primitive_part2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_is_monic(char * in, int out, int verbose)
{
    int ans;
    fmpq_poly_t op;
    int res;
    
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    res = fmpq_poly_is_monic(op);
    
    ans = res == out;
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_is_monic: passed\n");
        }
        else
        {
            printf("test_is_monic: failed\n");
            printf("    Expected \"%d\", got \"%d\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(op);
    
    return ans ? 1 : 0;
}

int test_resultant(char * in1, char * in2, char * out, int verbose)
{
    int ans;
    mpq_t rop;
    fmpq_poly_t op1, op2;
    char * res;
    
    mpq_init(rop);
    fmpq_poly_init(op1);
    fmpq_poly_init(op2);
    fmpq_poly_from_string(op1, in1);
    fmpq_poly_from_string(op2, in2);
    fmpq_poly_resultant(rop, op1, op2);
    
    res = mpq_get_str(NULL, 10, rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_resultant: passed\n");
        }
        else
        {
            printf("test_resultant: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    mpq_clear(rop);
    fmpq_poly_clear(op1);
    fmpq_poly_clear(op2);
    free(res);
    
    return ans ? 1 : 0;
}

int test_discriminant(char * in, char * out, int verbose)
{
    int ans;
    mpq_t rop;
    fmpq_poly_t op;
    char * res;
    
    mpq_init(rop);
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_discriminant(rop, op);
    
    res = mpq_get_str(NULL, 10, rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_discriminant: passed\n");
        }
        else
        {
            printf("test_discriminant: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    mpq_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

int test_rescale1(char * in, mpq_t a, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_rescale(rop, op, a);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_rescale1: passed\n");
        }
        else
        {
            printf("test_rescale1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

int test_rescale2(char * in, mpq_t a, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_rescale(rop, rop, a);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_rescale2: passed\n");
        }
        else
        {
            printf("test_rescale2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_is_squarefree(char * in, int out, int verbose)
{
    int ans;
    fmpq_poly_t op;
    int res;
    
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    
    res = fmpq_poly_is_squarefree(op);
    ans = res == out;
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_is_squarefree: passed\n");
        }
        else
        {
            printf("test_is_squarefree: failed\n");
            printf("    Expected \"%d\", got \"%d\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(op);
    
    return ans ? 1 : 0;
}

int test_getslice1(char * in, long i, long j, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_getslice(rop, op, i, j);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_getslice1: passed\n");
        }
        else
        {
            printf("test_getslice1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

int test_getslice2(char * in, long i, long j, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_getslice(rop, rop, i, j);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_getslice2: passed\n");
        }
        else
        {
            printf("test_getslice2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_left_shift1(char * in, ulong n, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(op);
    fmpq_poly_init(rop);
    fmpq_poly_from_string(op, in);
    fmpq_poly_left_shift(rop, op, n);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_left_shift1: passed\n");
        }
        else
        {
            printf("test_left_shift1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

int test_left_shift2(char * in, ulong n, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_left_shift(rop, rop, n);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_left_shift2: passed\n");
        }
        else
        {
            printf("test_left_shift2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_right_shift1(char * in, ulong n, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(op);
    fmpq_poly_init(rop);
    fmpq_poly_from_string(op, in);
    fmpq_poly_right_shift(rop, op, n);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_right_shift1: passed\n");
        }
        else
        {
            printf("test_right_shift1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

int test_right_shift2(char * in, ulong n, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_right_shift(rop, rop, n);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_right_shift2: passed\n");
        }
        else
        {
            printf("test_right_shift2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_truncate1(char * in, unsigned long n, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_truncate(rop, op, n);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_truncate1: passed\n");
        }
        else
        {
            printf("test_truncate1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

int test_truncate2(char * in, unsigned long n, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_truncate(rop, rop, n);
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_truncate2: passed\n");
        }
        else
        {
            printf("test_truncate2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_reverse1(char * in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop, op;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in);
    fmpq_poly_reverse(rop, op, fmpq_poly_length(op));
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_reverse1: passed\n");
        }
        else
        {
            printf("test_reverse1: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

int test_reverse2(char * in, char * out, int verbose)
{
    int ans;
    fmpq_poly_t rop;
    char * res;
    
    fmpq_poly_init(rop);
    fmpq_poly_from_string(rop, in);
    fmpq_poly_reverse(rop, rop, fmpq_poly_length(rop));
    
    res = fmpq_poly_to_string(rop);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_reverse2: passed\n");
        }
        else
        {
            printf("test_reverse2: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(rop);
    free(res);
    
    return ans ? 1 : 0;
}

int test_to_string_pretty(char * in1, char * in2, char * out, int verbose)
{
    int ans;
    fmpq_poly_t op;
    char * res;
    
    fmpq_poly_init(op);
    fmpq_poly_from_string(op, in1);
    
    res = fmpq_poly_to_string_pretty(op, in2);
    ans = !strcmp(res, out);
    
    if (verbose)
    {
        if (ans)
        {
            printf("test_to_string_pretty: passed\n");
        }
        else
        {
            printf("test_to_string_pretty: failed\n");
            printf("    Expected \"%s\", got \"%s\"\n", out, res);
        }
    }
    
    fmpq_poly_clear(op);
    free(res);
    
    return ans ? 1 : 0;
}

int main(int argc, char *argv[])
{
    // char * str;
    // char * strin;
    // char * strout;
    // int ans;
    
    // fmpq_poly_t poly1, poly2, poly3, poly4;
    // fmpz_poly_t polyz1;
    
    fmpz_t fmpz1, fmpz2, fmpz3;
    mpz_t mpz1, mpz2, mpz3;
    mpq_t mpq1, mpq2, mpq3;
    
    fmpz1 = fmpz_init(1);
    fmpz2 = fmpz_init(1);
    fmpz3 = fmpz_init(1);
    mpz_init(mpz1);
    mpz_init(mpz2);
    mpz_init(mpz3);
    mpq_init(mpq1);
    mpq_init(mpq2);
    mpq_init(mpq3);
    
    //-----------------------------------------------------------------------//
    // Canonicalize                                                          //
    //-----------------------------------------------------------------------//
    
    fmpz_set_si(fmpz1, 1);
    fmpz_set_si(fmpz2, 2);
    fmpz_set_si(fmpz3, -1);
    test_canonicalize("2  2 1", fmpz1, "2  2 1", 1);
    test_canonicalize("2  2 1", fmpz2, "2  1 1/2", 1);
    test_canonicalize("2  2 1", fmpz3, "2  -2 -1", 1);
    
    //-----------------------------------------------------------------------//
    // Accessing numerator and denominator                                   //
    //-----------------------------------------------------------------------//
    
    test_numref("2  1/2 -1", "2  1 -2", 1);
    test_denref("2  1/3 -4/7", "21", 1);
    
    //-----------------------------------------------------------------------//
    // Assignment and basic manipulation                                     //
    //-----------------------------------------------------------------------//
    
    test_set("0", "0", 1);
    test_set("1  -1", "1  -1", 1);
    test_set("3  2/1 -4/2 28/7", "3  2 -2 4", 1);
    
    test_set_si(0, "0", 1);
    test_set_si(-2, "1  -2", 1);
    
    fmpz_set_si(fmpz1, 0);
    fmpz_set_si(fmpz2, -1);
    test_set_fmpz(fmpz1, "0", 1);
    test_set_fmpz(fmpz2, "1  -1", 1);
    
    mpz_set_si(mpz1, 0);
    mpz_set_si(mpz2, 23);
    test_set_mpz(mpz1, "0", 1);
    test_set_mpz(mpz2, "1  23", 1);
    
    test_swap("0", "2  1/3 -4/7", "2  1/3 -4/7", "0", 1);
    test_swap("1  1/2", "3  2/3 -2 7/2", "3  2/3 -2 7/2", "1  1/2", 1);
    
    test_zero("0", "0", 1);
    test_zero("1  -1", "0", 1);
    test_zero("3  2/1 -4/2 28/7", "0", 1);
    
    test_one("0", "1  1", 1);
    test_one("1  -1", "1  1", 1);
    test_one("3  2/1 -4/2 28/7", "1  1", 1);
    
    test_neg1("0", "0", 1);
    test_neg1("1  2/3", "1  -2/3", 1);
    test_neg2("0", "0", 1);
    test_neg2("1  2/3", "1  -2/3", 1);
    
    test_inv1("1  1", "1  1", 1);
    test_inv1("1  2/3", "1  3/2", 1);
    test_inv1("1  -2/5", "1  -5/2", 1);
    test_inv1("1  -1/7", "1  -7", 1);
    test_inv1("1  1/7", "1  7", 1);
    test_inv1("1  -3", "1  -1/3", 1);
    test_inv1("1  3", "1  1/3", 1);
    test_inv2("1  1", "1  1", 1);
    test_inv2("1  -2/3", "1  -3/2", 1);
    test_inv2("1  2/3", "1  3/2", 1);
    test_inv2("1  -2/5", "1  -5/2", 1);
    test_inv2("1  -1/7", "1  -7", 1);
    test_inv2("1  1/7", "1  7", 1);
    test_inv2("1  -3", "1  -1/3", 1);
    test_inv2("1  3", "1  1/3", 1);
    
    //-----------------------------------------------------------------------//
    // Setting/ retrieving individual coefficients                           //
    //-----------------------------------------------------------------------//
    
    test_get_coeff_mpq("0", 0, "0", 1);
    test_get_coeff_mpq("0", 10, "0", 1);
    test_get_coeff_mpq("3  2/1 -4/2 27/7", 0, "2", 1);
    test_get_coeff_mpq("3  2/1 -4/2 27/7", 1, "-2", 1);
    test_get_coeff_mpq("3  2/1 -4/2 27/7", 2, "27/7", 1);
    
    fmpz_set_si(fmpz1, 0);
    fmpz_set_si(fmpz2, -2);
    test_set_coeff_fmpz("0", 1, fmpz1, "0", 1);
    test_set_coeff_fmpz("0", 2, fmpz2, "3  0 0 -2", 1);
    test_set_coeff_fmpz("1  2", 0, fmpz1, "0", 1);
    test_set_coeff_fmpz("1  2", 2, fmpz2, "3  2 0 -2", 1);
    test_set_coeff_fmpz("3  -1/2 0 2/3", 1, fmpz2, "3  -1/2 -2 2/3", 1);
    test_set_coeff_fmpz("3  -1/2 2 2/3", 1, fmpz2, "3  -1/2 -2 2/3", 1);
    
    mpz_set_si(mpz1, 0);
    mpz_set_si(mpz2, -2);
    test_set_coeff_mpz("0", 1, mpz1, "0", 1);
    test_set_coeff_mpz("0", 2, mpz2, "3  0 0 -2", 1);
    test_set_coeff_mpz("1  2", 0, mpz1, "0", 1);
    test_set_coeff_mpz("1  2", 2, mpz2, "3  2 0 -2", 1);
    test_set_coeff_mpz("3  -1/2 0 2/3", 1, mpz2, "3  -1/2 -2 2/3", 1);
    test_set_coeff_mpz("3  -1/2 2 2/3", 1, mpz2, "3  -1/2 -2 2/3", 1);
    
    test_set_coeff_si("0", 1, 0, "0", 1);
    test_set_coeff_si("0", 2, -2, "3  0 0 -2", 1);
    test_set_coeff_si("1  2", 0, 0, "0", 1);
    test_set_coeff_si("1  2", 2, -2, "3  2 0 -2", 1);
    test_set_coeff_si("3  -1/2 0 2/3", 1, -2, "3  -1/2 -2 2/3", 1);
    test_set_coeff_si("3  -1/2 2 2/3", 1, -2, "3  -1/2 -2 2/3", 1);
    
    mpq_set_si(mpq1, 0, 1);
    mpq_set_si(mpq2, -1, 2);
    test_set_coeff_mpq("0", 1, mpq1, "0", 1);
    test_set_coeff_mpq("1  2", 0, mpq1, "0", 1);
    test_set_coeff_mpq("2  1/3 -4/7", 1, mpq1, "1  1/3", 1);
    test_set_coeff_mpq("2  1/3 -4/7", 1, mpq2, "2  1/3 -1/2", 1);
    test_set_coeff_mpq("3  1/2 0 2", 1, mpq2, "3  1/2 -1/2 2", 1);
    test_set_coeff_mpq("3  1/2 -3 2", 1, mpq2, "3  1/2 -1/2 2", 1);
    
    //-----------------------------------------------------------------------//
    // Comparison                                                            //
    //-----------------------------------------------------------------------//
    
    test_equal("0", "0", 1, 1);
    test_equal("0", "2  2/3 -3/4", 0, 1);
    test_equal("2  1/2 3/2", "2  5/2 3/2", 0, 1);
    test_equal("2  1/2 3/2", "2  5 3", 0, 1);
    
    test_cmp("0", "0", 0, 1);
    test_cmp("2  1/3 -4/7", "2  1/3 -4/7", 0, 1);
    test_cmp("2  1/3 -4/7", "1  2/3", 1, 1);
    test_cmp("1  2/3", "2  1/3 -4/7", -1, 1);
    test_cmp("3  1 2 3", "3  1 2 4", -1, 1);
    test_cmp("3  1 2 3/2", "3  1 2 4", -1, 1);
    test_cmp("3  1 2 3", "3  1 2 3/2", 1, 1);
    test_cmp("3  1 2 3", "3  1 2 7/2", -1, 1);
    test_cmp("3  1 2 3", "3  1 3/2 3", 1, 1);
    test_cmp("3  1/4 2 3", "3  1 2 3", -1, 1);
    test_cmp("3  3/4 2 1", "3  1/5 2 1", 1, 1);
    
    //-----------------------------------------------------------------------//
    // Polynomial parameters                                                 //
    //-----------------------------------------------------------------------//
    
    test_length("0", 0, 1);
    test_length("1  2/3", 1, 1);
    test_length("2  1/2 -3/4", 2, 1);
    
    test_degree("0", -1, 1);
    test_degree("1  2/3", 0, 1);
    test_degree("2  1/2 -3/4", 1, 1);
    
    //-----------------------------------------------------------------------//
    // Addition/ subtraction                                                 //
    //-----------------------------------------------------------------------//
    
    test_binop1(add, "5  0 1/3 4/5 0 11", "4  -2 0 0 1/3", "5  -2 1/3 4/5 1/3 11", 1);
    test_binop1(add, "2  1 -2", "3  -1 0 1", "3  0 -2 1", 1);
    test_binop1(add, "2  1 -2", "3  -1 1/2 2", "3  0 -3/2 2", 1);
    test_binop1(add, "3  -1 1/2 2", "2  1 -2", "3  0 -3/2 2", 1);
    
    test_binop2(add, "5  0 1/3 4/5 0 11", "5  0 2/3 8/5 0 22", 1);
    test_binop2(add, "4  -2 0 0 1/3", "4  -4 0 0 2/3", 1);
    
    test_binop3(add, "5  0 1/3 4/5 0 11", "4  -2 0 0 1/3", "5  -2 1/3 4/5 1/3 11", 1);
    test_binop3(add, "4  -2 0 0 1/3", "5  0 1/3 4/5 0 11", "5  -2 1/3 4/5 1/3 11", 1);
    test_binop3(add, "4  -2 0 0 1/3", "0", "4  -2 0 0 1/3", 1);
    test_binop3(add, "0", "2  -1/2 3", "2  -1/2 3", 1);
    
    test_binop4(add, "5  0 1/3 4/5 0 11", "4  -2 0 0 1/3", "5  -2 1/3 4/5 1/3 11", 1);
    test_binop4(add, "4  -2 0 0 1/3", "5  0 1/3 4/5 0 11", "5  -2 1/3 4/5 1/3 11", 1);
    
    test_binop5(add, "5  0 1/3 4/5 0 11", "5  0 2/3 8/5 0 22", 1);
    test_binop5(add, "4  -2 0 0 1/3", "4  -4 0 0 2/3", 1);
    
    test_binop6(add, "2  -1/2 2", "3  -1 1 2", "2  1 -2", "3  0 -1 2", 1);
    
    test_binop1(sub, "5  0 1/3 4/5 0 11", "4  -2 0 0 1/3", "5  2 1/3 4/5 -1/3 11", 1);
    test_binop1(sub, "2  1 -2", "3  -1 0 1", "3  2 -2 -1", 1);
    test_binop1(sub, "2  1 -2", "3  -1 1/2 2", "3  2 -5/2 -2", 1);
    test_binop1(sub, "3  -1 1/2 2", "2  1 -2", "3  -2 5/2 2", 1);
    
    test_binop2(sub, "4  -2 0 0 1/3", "0", 1);
    test_binop2(sub, "5  2 1/3 4/5 -1/3 11", "0", 1);
    
    test_binop3(sub, "5  0 1/3 4/5 0 11", "4  -2 0 0 1/3", "5  2 1/3 4/5 -1/3 11", 1);
    test_binop3(sub, "4  -2 0 0 1/3", "5  0 1/3 4/5 0 11", "5  -2 -1/3 -4/5 1/3 -11", 1);
    
    test_binop4(sub, "5  0 1/3 4/5 0 11", "4  -2 0 0 1/3", "5  2 1/3 4/5 -1/3 11", 1);
    test_binop4(sub, "4  -2 0 0 1/3", "5  0 1/3 4/5 0 11", "5  -2 -1/3 -4/5 1/3 -11", 1);
    
    test_binop5(sub, "4  -2 0 0 1/3", "0", 1);
    test_binop5(sub, "5  2 1/3 4/5 -1/3 11", "0", 1);
    
    test_binop6(sub, "2, -1/2 2", "3  -1 1 2", "2  1 -2", "3  -2 3 2", 1);
    
    test_binop1(addmul, "2  1 -2", "3  -1 1/2 2", "4  -1 5/2 1 -4", 1);
    
    test_binop1(submul, "2  1 -2", "3  -1 1/2 2", "4  1 -5/2 -1 4", 1);
    
    //-----------------------------------------------------------------------//
    // Scalar multiplication and division                                    //
    //-----------------------------------------------------------------------//
    
    // Scalar multiplication
    
    test_scalar_mul_si1("5  0 1/3 4/5 0 11", 1, "5  0 1/3 4/5 0 11", 1);
    test_scalar_mul_si1("5  0 1/3 4/5 0 11", -1, "5  0 -1/3 -4/5 0 -11", 1);
    test_scalar_mul_si1("5  0 1/3 4/5 0 11", 3, "5  0 1 12/5 0 33", 1);
    test_scalar_mul_si1("2  -1 2", 3, "2  -3 6", 1);
    
    test_scalar_mul_si2("5  0 1/3 4/5 0 11", 1, "5  0 1/3 4/5 0 11", 1);
    test_scalar_mul_si2("5  0 1/3 4/5 0 11", -1, "5  0 -1/3 -4/5 0 -11", 1);
    test_scalar_mul_si2("5  0 1/3 4/5 0 11", 3, "5  0 1 12/5 0 33", 1);
    test_scalar_mul_si2("2  -1 2", 3, "2  -3 6", 1);
    
    mpz_set_si(mpz1, 1);
    mpz_set_si(mpz2, -1);
    mpz_set_si(mpz3, 3);
    
    test_scalar_mul_mpz1("5  0 1/3 4/5 0 11", mpz1, "5  0 1/3 4/5 0 11", 1);
    test_scalar_mul_mpz1("5  0 1/3 4/5 0 11", mpz2, "5  0 -1/3 -4/5 0 -11", 1);
    test_scalar_mul_mpz1("5  0 1/3 4/5 0 11", mpz3, "5  0 1 12/5 0 33", 1);
    test_scalar_mul_mpz1("2  -1 2", mpz3, "2  -3 6", 1);
    
    test_scalar_mul_mpz2("5  0 1/3 4/5 0 11", mpz1, "5  0 1/3 4/5 0 11", 1);
    test_scalar_mul_mpz2("5  0 1/3 4/5 0 11", mpz2, "5  0 -1/3 -4/5 0 -11", 1);
    test_scalar_mul_mpz2("5  0 1/3 4/5 0 11", mpz3, "5  0 1 12/5 0 33", 1);
    test_scalar_mul_mpz2("2  -1 2", mpz3, "2  -3 6", 1);
    
    mpq_set_si(mpq1, 1, 1);
    mpq_set_si(mpq2, -1, 1);
    mpq_set_si(mpq3, 3, 1);
    
    test_scalar_mul_mpq1("5  0 1/3 4/5 0 11", mpq1, "5  0 1/3 4/5 0 11", 1);
    test_scalar_mul_mpq1("5  0 1/3 4/5 0 11", mpq2, "5  0 -1/3 -4/5 0 -11", 1);
    test_scalar_mul_mpq1("5  0 1/3 4/5 0 11", mpq3, "5  0 1 12/5 0 33", 1);
    test_scalar_mul_mpq1("2  -1 2", mpq3, "2  -3 6", 1);
    
    test_scalar_mul_mpq2("5  0 1/3 4/5 0 11", mpq1, "5  0 1/3 4/5 0 11", 1);
    test_scalar_mul_mpq2("5  0 1/3 4/5 0 11", mpq2, "5  0 -1/3 -4/5 0 -11", 1);
    test_scalar_mul_mpq2("5  0 1/3 4/5 0 11", mpq3, "5  0 1 12/5 0 33", 1);
    test_scalar_mul_mpq2("2  -1 2", mpq3, "2  -3 6", 1);
    
    mpq_set_si(mpq1, 1, 7);
    mpq_set_si(mpq2, -3, 4);
    mpq_set_si(mpq3, -4, 5);
    
    test_scalar_mul_mpq1("5  0 1/3 4/5 0 11", mpq1, "5  0 1/21 4/35 0 11/7", 1);
    test_scalar_mul_mpq1("5  0 1/3 4/5 0 11", mpq2, "5  0 -1/4 -3/5 0 -33/4", 1);
    test_scalar_mul_mpq1("5  0 1/3 4/5 0 11", mpq3, "5  0 -4/15 -16/25 0 -44/5", 1);
    
    test_scalar_mul_mpq2("5  0 1/3 4/5 0 11", mpq1, "5  0 1/21 4/35 0 11/7", 1);
    test_scalar_mul_mpq2("5  0 1/3 4/5 0 11", mpq2, "5  0 -1/4 -3/5 0 -33/4", 1);
    test_scalar_mul_mpq2("5  0 1/3 4/5 0 11", mpq3, "5  0 -4/15 -16/25 0 -44/5", 1);
    
    // Scalar division
    
    test_scalar_div_si1("5  0 1/3 4/5 0 11", 1, "5  0 1/3 4/5 0 11", 1);
    test_scalar_div_si1("5  0 1/3 4/5 0 11", -1, "5  0 -1/3 -4/5 0 -11", 1);
    test_scalar_div_si1("5  0 1/3 4/5 0 11", 3, "5  0 1/9 4/15 0 11/3", 1);
    test_scalar_div_si1("3  2/3 -4/3 22/3", 2, "3  1/3 -2/3 11/3", 1);
    test_scalar_div_si1("3  2/3 -4/3 22/3", 3, "3  2/9 -4/9 22/9", 1);
    test_scalar_div_si1("3  2/3 -4/3 22/3", -2, "3  -1/3 2/3 -11/3", 1);
    test_scalar_div_si1("3  2/3 -4/3 22/3", -3, "3  -2/9 4/9 -22/9", 1);
    test_scalar_div_si1("3  2 -4 22", 2, "3  1 -2 11", 1);
    test_scalar_div_si1("3  1 -4 22", 2, "3  1/2 -2 11", 1);
    test_scalar_div_si1("3  1 -4 22", -2, "3  -1/2 2 -11", 1);
    test_scalar_div_si1("3  2 4 6", 3, "3  2/3 4/3 2", 1);
    test_scalar_div_si1("3  2 4 6", -3, "3  -2/3 -4/3 -2", 1);
    
    test_scalar_div_si2("5  0 1/3 4/5 0 11", 1, "5  0 1/3 4/5 0 11", 1);
    test_scalar_div_si2("5  0 1/3 4/5 0 11", -1, "5  0 -1/3 -4/5 0 -11", 1);
    test_scalar_div_si2("5  0 1/3 4/5 0 11", 3, "5  0 1/9 4/15 0 11/3", 1);
    test_scalar_div_si2("3  2/3 -4/3 22/3", 2, "3  1/3 -2/3 11/3", 1);
    test_scalar_div_si2("3  2/3 -4/3 22/3", 3, "3  2/9 -4/9 22/9", 1);
    test_scalar_div_si2("3  2/3 -4/3 22/3", -2, "3  -1/3 2/3 -11/3", 1);
    test_scalar_div_si2("3  2/3 -4/3 22/3", -3, "3  -2/9 4/9 -22/9", 1);
    test_scalar_div_si2("3  2 -4 22", 2, "3  1 -2 11", 1);
    test_scalar_div_si2("3  1 -4 22", 2, "3  1/2 -2 11", 1);
    test_scalar_div_si2("3  1 -4 22", -2, "3  -1/2 2 -11", 1);
    test_scalar_div_si2("3  2 4 6", 3, "3  2/3 4/3 2", 1);
    test_scalar_div_si2("3  2 4 6", -3, "3  -2/3 -4/3 -2", 1);
    
    mpz_set_si(mpz1, 1);
    mpz_set_si(mpz2, -1);
    mpz_set_si(mpz3, 3);
    
    test_scalar_div_mpz1("5  0 1/3 4/5 0 11", mpz1, "5  0 1/3 4/5 0 11", 1);
    test_scalar_div_mpz1("5  0 1/3 4/5 0 11", mpz2, "5  0 -1/3 -4/5 0 -11", 1);
    test_scalar_div_mpz1("5  0 1/3 4/5 0 11", mpz3, "5  0 1/9 4/15 0 11/3", 1);
    test_scalar_div_mpz1("3  2/3 -4/3 22/3", mpz3, "3  2/9 -4/9 22/9", 1);
    
    test_scalar_div_mpz2("5  0 1/3 4/5 0 11", mpz1, "5  0 1/3 4/5 0 11", 1);
    test_scalar_div_mpz2("5  0 1/3 4/5 0 11", mpz2, "5  0 -1/3 -4/5 0 -11", 1);
    test_scalar_div_mpz2("5  0 1/3 4/5 0 11", mpz3, "5  0 1/9 4/15 0 11/3", 1);
    test_scalar_div_mpz2("3  2/3 -4/3 22/3", mpz3, "3  2/9 -4/9 22/9", 1);
    
    mpz_set_si(mpz1, 2);
    mpz_set_si(mpz2, 3);
    
    test_scalar_div_mpz1("3  2/3 -4/3 22/3", mpz2, "3  2/9 -4/9 22/9", 1);
    test_scalar_div_mpz1("3  2 -4 22", mpz1, "3  1 -2 11", 1);
    test_scalar_div_mpz1("3  2 -4 22", mpz1, "3  1 -2 11", 1);
    test_scalar_div_mpz1("3  1 -4 22", mpz1, "3  1/2 -2 11", 1);
    test_scalar_div_mpz1("3  2 4 6", mpz2, "3  2/3 4/3 2", 1);
    
    test_scalar_div_mpz2("3  2/3 -4/3 22/3", mpz2, "3  2/9 -4/9 22/9", 1);
    test_scalar_div_mpz2("3  2 -4 22", mpz1, "3  1 -2 11", 1);
    test_scalar_div_mpz2("3  2 -4 22", mpz1, "3  1 -2 11", 1);
    test_scalar_div_mpz2("3  1 -4 22", mpz1, "3  1/2 -2 11", 1);
    test_scalar_div_mpz2("3  2 4 6", mpz2, "3  2/3 4/3 2", 1);
    
    mpz_set_si(mpz1, -2);
    mpz_set_si(mpz2, -3);
    
    test_scalar_div_mpz1("3  2/3 -4/3 22/3", mpz1, "3  -1/3 2/3 -11/3", 1);
    test_scalar_div_mpz1("3  2/3 -4/3 22/3", mpz2, "3  -2/9 4/9 -22/9", 1);
    test_scalar_div_mpz1("3  1 -4 22", mpz1, "3  -1/2 2 -11", 1);
    test_scalar_div_mpz1("3  2 4 6", mpz2, "3  -2/3 -4/3 -2", 1);
    
    test_scalar_div_mpz2("3  2/3 -4/3 22/3", mpz1, "3  -1/3 2/3 -11/3", 1);
    test_scalar_div_mpz2("3  2/3 -4/3 22/3", mpz2, "3  -2/9 4/9 -22/9", 1);
    test_scalar_div_mpz2("3  1 -4 22", mpz1, "3  -1/2 2 -11", 1);
    test_scalar_div_mpz2("3  2 4 6", mpz2, "3  -2/3 -4/3 -2", 1);
    
    mpq_set_si(mpq1, 1, 1);
    mpq_set_si(mpq2, -1, 1);
    mpq_set_si(mpq3, 3, 1);
    
    test_scalar_div_mpq1("5  0 1/3 4/5 0 11", mpq1, "5  0 1/3 4/5 0 11", 1);
    test_scalar_div_mpq1("5  0 1/3 4/5 0 11", mpq2, "5  0 -1/3 -4/5 0 -11", 1);
    test_scalar_div_mpq1("5  0 1/3 4/5 0 11", mpq3, "5  0 1/9 4/15 0 11/3", 1);
    test_scalar_div_mpq1("3  2/3 -4/3 22/3", mpq3, "3  2/9 -4/9 22/9", 1);
    test_scalar_div_mpq1("3  1 3 2", mpq3, "3  1/3 1 2/3", 1);
    
    test_scalar_div_mpq2("5  0 1/3 4/5 0 11", mpq1, "5  0 1/3 4/5 0 11", 1);
    test_scalar_div_mpq2("5  0 1/3 4/5 0 11", mpq2, "5  0 -1/3 -4/5 0 -11", 1);
    test_scalar_div_mpq2("5  0 1/3 4/5 0 11", mpq3, "5  0 1/9 4/15 0 11/3", 1);
    test_scalar_div_mpq2("3  2/3 -4/3 22/3", mpq3, "3  2/9 -4/9 22/9", 1);
    test_scalar_div_mpq2("3  1 3 2", mpq3, "3  1/3 1 2/3", 1);
    
    mpq_set_si(mpq1, 1, 7);
    mpq_set_si(mpq2, -3, 4);
    mpq_set_si(mpq3, -4, 5);
    
    test_scalar_div_mpq1("5  0 1/3 4/5 0 11", mpq1, "5  0 7/3 28/5 0 77", 1);
    test_scalar_div_mpq1("5  0 1/3 4/5 0 11", mpq2, "5  0 -4/9 -16/15 0 -44/3", 1);
    test_scalar_div_mpq1("5  0 1/3 4/5 0 11", mpq3, "5  0 -5/12 -1 0 -55/4", 1);
    
    test_scalar_div_mpq2("5  0 1/3 4/5 0 11", mpq1, "5  0 7/3 28/5 0 77", 1);
    test_scalar_div_mpq2("5  0 1/3 4/5 0 11", mpq2, "5  0 -4/9 -16/15 0 -44/3", 1);
    test_scalar_div_mpq2("5  0 1/3 4/5 0 11", mpq3, "5  0 -5/12 -1 0 -55/4", 1);
    
    //-----------------------------------------------------------------------//
    // Multiplication                                                        //
    //-----------------------------------------------------------------------//
    
    test_binop1(mul, "5  0 1/3 4/5 0 11", "4  -2 0 0 1/3", "8  0 -2/3 -8/5 0 -197/9 4/15 0 11/3", 1);
    test_binop1(mul, "0", "2  -3/2 1", "0", 1);
    test_binop1(mul, "2  -3/2 1", "0", "0", 1);
    test_binop1(mul, "2  1 2", "2  -2 3", "3  -2 -1 6", 1);
    test_binop1(mul, "2  2 2", "2  1/3 2/3", "3  2/3 2 4/3", 1);
    test_binop1(mul, "3  1/2 3/2 1", "4  0 3/4 0 3/4", "6  0 3/8 9/8 9/8 9/8 3/4", 1);
    test_binop1(mul, "3  1/2 3/2 1", "3  0 2/3 4/3", "5  0 1/3 5/3 8/3 4/3", 1);
    test_binop1(mul, "2  2/3 2/3", "2  1 3/2", "3  2/3 5/3 1", 1);
    test_binop1(mul, "2  2/3 2/3", "2  4/5 4/5", "3  8/15 16/15 8/15", 1);
    test_binop1(mul, "2  2/3 2/3", "2  5/2 15/2", "3  5/3 20/3 5", 1);
    test_binop1(mul, "2  2/3 2/3", "2  5/3 5", "3  10/9 40/9 10/3", 1);
    test_binop1(mul, "2  2/5 2/5", "2  5/2 15/2", "3  1 4 3", 1);
    test_binop1(mul, "2  2/5 2/5", "2  2/3 1", "3  4/15 2/3 2/5", 1);
    test_binop1(mul, "2  2/5 2/5", "2  5/3 5/3", "3  2/3 4/3 2/3", 1);
    test_binop1(mul, "2  5/3 5/3", "2  2/5 2/5", "3  2/3 4/3 2/3", 1);
    
    test_binop2(mul, "5  0 1/3 4/5 0 11", "9  0 0 1/9 8/15 16/25 22/3 88/5 0 121", 1);
    test_binop2(mul, "4  -2 0 0 1/3", "7  4 0 0 -4/3 0 0 1/9", 1);
    test_binop2(mul, "0", "0", 1);
    
    test_binop3(mul, "5  0 1/3 4/5 0 11", "4  -2 0 0 1/3", "8  0 -2/3 -8/5 0 -197/9 4/15 0 11/3", 1);
    test_binop3(mul, "4  -2 0 0 1/3", "5  0 1/3 4/5 0 11", "8  0 -2/3 -8/5 0 -197/9 4/15 0 11/3", 1);
    test_binop3(mul, "0", "2  -3/2 1", "0", 1);
    test_binop3(mul, "2  -3/2 1", "0", "0", 1);
    
    test_binop4(mul, "5  0 1/3 4/5 0 11", "4  -2 0 0 1/3", "8  0 -2/3 -8/5 0 -197/9 4/15 0 11/3", 1);
    test_binop4(mul, "4  -2 0 0 1/3", "5  0 1/3 4/5 0 11", "8  0 -2/3 -8/5 0 -197/9 4/15 0 11/3", 1);
    test_binop4(mul, "0", "2  -3/2 1", "0", 1);
    test_binop4(mul, "0", "2  -3/2 1", "0", 1);
    
    test_binop5(mul, "5  0 1/3 4/5 0 11", "9  0 0 1/9 8/15 16/25 22/3 88/5 0 121", 1);
    test_binop5(mul, "4  -2 0 0 1/3", "7  4 0 0 -4/3 0 0 1/9", 1);
    test_binop5(mul, "0", "0", 1);
    
    //-----------------------------------------------------------------------//
    // Division                                                              //
    //-----------------------------------------------------------------------//
    
    test_binop1(floordiv, "1  1", "1  1", "1  1", 1);
    test_binop1(floordiv, "5  0 1/3 4/5 0 11", "2  3/2 2", "4  -4711/1920 559/160 -33/8 11/2", 1);
    test_binop1(floordiv, "5  1 -8 1/2 0 1/4", "3  1 -4 1", "3  17/4 1 1/4", 1);
    test_binop1(floordiv, "5  11 0 -9/28 -1/2 -12", "3  -1/3 1/10 5", "3  -9767/43750 -13/250 -12/5", 1);
    test_binop1(floordiv, "2  3 2", "1  2", "2  3/2 1", 1);
    test_binop1(floordiv, "2  3 2", "1  2/3", "2  9/2 3", 1);
    test_binop1(floordiv, "2  3/5 2/5", "1  2", "2  3/10 1/5", 1);
    test_binop1(floordiv, "2  3/5 2/5", "1  2/3", "2  9/10 3/5", 1);
    
    test_binop2(floordiv, "1  2", "1  1", 1);
    test_binop2(floordiv, "3  1/2 -3/4 1", "1  1", 1);
    
    test_binop3(floordiv, "5  0 1/3 4/5 0 11", "2  3/2 2", "4  -4711/1920 559/160 -33/8 11/2", 1);
    test_binop3(floordiv, "5  1 -8 1/2 0 1/4", "3  1 -4 1", "3  17/4 1 1/4", 1);
    test_binop3(floordiv, "5  11 0 -9/28 -1/2 -12", "3  -1/3 1/10 5", "3  -9767/43750 -13/250 -12/5", 1);
    test_binop3(floordiv, "2  3 2", "1  2", "2  3/2 1", 1);
    test_binop3(floordiv, "2  3 2", "1  2/3", "2  9/2 3", 1);
    test_binop3(floordiv, "2  3/5 2/5", "1  2", "2  3/10 1/5", 1);
    test_binop3(floordiv, "2  3/5 2/5", "1  2/3", "2  9/10 3/5", 1);
    
    test_binop1(mod, "5  0 1/3 4/5 0 11", "2  3/2 2", "1  4711/1280", 1);
    test_binop1(mod, "5  11 0 -9/28 -1/2 -12", "3  -1/3 1/10 5", "2  1433983/131250 6551/1312500", 1);
    test_binop1(mod, "3  1 2 -3/2", "1  2", "0", 1);
    
    test_binop2(mod, "1  2", "0", 1);
    test_binop2(mod, "3  1/2 -3/4 1", "0", 1);
    
    test_binop3(mod, "5  0 1/3 4/5 0 11", "2  3/2 2", "1  4711/1280", 1);
    test_binop3(mod, "5  11 0 -9/28 -1/2 -12", "3  -1/3 1/10 5", "2  1433983/131250 6551/1312500", 1);
    
    test_binop4(mod, "5  0 1/3 4/5 0 11", "2  3/2 2", "1  4711/1280", 1);
    test_binop4(mod, "5  11 0 -9/28 -1/2 -12", "3  -1/3 1/10 5", "2  1433983/131250 6551/1312500", 1);
     
    //-----------------------------------------------------------------------//
    // Powering                                                              //
    //-----------------------------------------------------------------------//
    
    test_power1("2  1/2 2", 0, "1  1", 1);
    test_power1("0", 0, "1  1", 1);
    test_power1("2  2 3", 2, "3  4 12 9", 1);
    test_power1("2  1/3 2", 2, "3  1/9 4/3 4", 1);
    
    test_power2("2  1/2 2", 0, "1  1", 1);
    test_power2("0", 0, "1  1", 1);
    test_power2("2  2 3", 2, "3  4 12 9", 1);
    test_power2("2  1/3 2", 2, "3  1/9 4/3 4", 1);
    
    //-----------------------------------------------------------------------//
    // Greatest common divisor                                               //
    //-----------------------------------------------------------------------//
    
    test_binop1(gcd, "0", "0", "0", 1);
    test_binop1(gcd, "2  1/3 2", "0", "2  1/6 1", 1);
    test_binop1(gcd, "0", "2  1/3 2", "2  1/6 1", 1);
    test_binop1(gcd, "2  1/3 -2", "0", "2  -1/6 1", 1);
    test_binop1(gcd, "0", "2  1/3 -2", "2  -1/6 1", 1);
    
    test_binop1(gcd, "4  -6 7 0 -1", "4  10 -7 3 -1", "2  -2 1", 1);
    test_binop1(gcd, "4  -6 7 0 -1", "4  -10 7 -3 1", "2  -2 1", 1);
    test_binop1(gcd, "4  6 -7 0 1", "4  -10 7 -3 1", "2  -2 1", 1);
    test_binop1(gcd, "4  6 -7 0 1", "4  10 -7 3 -1", "2  -2 1", 1);
    
    test_binop3(gcd, "0", "0", "0", 1);
    test_binop3(gcd, "2  1/3 2", "0", "2  1/6 1", 1);
    test_binop3(gcd, "0", "2  1/3 2", "2  1/6 1", 1);
    test_binop3(gcd, "2  1/3 -2", "0", "2  -1/6 1", 1);
    test_binop3(gcd, "0", "2  1/3 -2", "2  -1/6 1", 1);
    
    test_binop3(gcd, "4  -6 7 0 -1", "4  10 -7 3 -1", "2  -2 1", 1);
    test_binop3(gcd, "4  -6 7 0 -1", "4  -10 7 -3 1", "2  -2 1", 1);
    test_binop3(gcd, "4  6 -7 0 1", "4  -10 7 -3 1", "2  -2 1", 1);
    test_binop3(gcd, "4  6 -7 0 1", "4  10 -7 3 -1", "2  -2 1", 1);
    
    test_binop4(gcd, "0", "0", "0", 1);
    test_binop4(gcd, "2  1/3 2", "0", "2  1/6 1", 1);
    test_binop4(gcd, "0", "2  1/3 2", "2  1/6 1", 1);
    test_binop4(gcd, "2  1/3 -2", "0", "2  -1/6 1", 1);
    test_binop4(gcd, "0", "2  1/3 -2", "2  -1/6 1", 1);
    
    test_binop4(gcd, "4  -6 7 0 -1", "4  10 -7 3 -1", "2  -2 1", 1);
    test_binop4(gcd, "4  -6 7 0 -1", "4  -10 7 -3 1", "2  -2 1", 1);
    test_binop4(gcd, "4  6 -7 0 1", "4  -10 7 -3 1", "2  -2 1", 1);
    test_binop4(gcd, "4  6 -7 0 1", "4  10 -7 3 -1", "2  -2 1", 1);
    
    test_xgcd1("0", "0", "0", "0", "0", 1);
    test_xgcd1("0", "1  1", "1  1", "0", "1  1", 1);
    test_xgcd1("0", "2  2 3", "2  2/3 1", "0", "1  1/3", 1);
    test_xgcd1("0", "2  2 -3", "2  -2/3 1", "0", "1  -1/3", 1);
    test_xgcd1("0", "2  1 1/2", "2  2 1", "0", "1  2", 1);
    test_xgcd1("0", "2  1 3/2", "2  2/3 1", "0", "1  2/3", 1);
    test_xgcd1("0", "2  -1 -1/2", "2  2 1", "0", "1  -2", 1);
    test_xgcd1("0", "2  -1 -3/2", "2  2/3 1", "0", "1  -2/3", 1);
    test_xgcd1("2  2 -1", "3  1 2 -3", "1  1", "2  4/7 3/7", "1  -1/7", 1);
    test_xgcd1("2  2/3 1/2", "3  1 2/3 1", "1  1", "2  12/17 -18/17", "1  9/17", 1);
    
    test_xgcd1("1  1", "0", "1  1", "1  1", "0", 1);
    test_xgcd1("2  2 3", "0", "2  2/3 1", "1  1/3", "0", 1);
    test_xgcd1("2  2 -3", "0", "2  -2/3 1", "1  -1/3", "0", 1);
    test_xgcd1("2  1 1/2", "0", "2  2 1", "1  2", "0", 1);
    test_xgcd1("2  1 3/2", "0", "2  2/3 1", "1  2/3", "0", 1);
    test_xgcd1("2  -1 -1/2", "0", "2  2 1", "1  -2", "0", 1);
    test_xgcd1("2  -1 -3/2", "0", "2  2/3 1", "1  -2/3", "0", 1);
    
    test_xgcd1("3  4 0 -1", "4  2 1 1 -1", "2  -2 1", "2  -1/3 1/3", "1  -1/3", 1);
    test_xgcd1("3  4 -4 -3", "4  2 -1 -1 -3", "2  -2/3 1", "2  -1/9 1/9", "1  -1/9", 1);
    
    test_xgcd2("0", "0", "0", "0", "0", 1);
    test_xgcd2("0", "1  1", "1  1", "0", "1  1", 1);
    test_xgcd2("0", "2  2 3", "2  2/3 1", "0", "1  1/3", 1);
    test_xgcd2("0", "2  2 -3", "2  -2/3 1", "0", "1  -1/3", 1);
    test_xgcd2("0", "2  1 1/2", "2  2 1", "0", "1  2", 1);
    test_xgcd2("0", "2  1 3/2", "2  2/3 1", "0", "1  2/3", 1);
    test_xgcd2("0", "2  -1 -1/2", "2  2 1", "0", "1  -2", 1);
    test_xgcd2("0", "2  -1 -3/2", "2  2/3 1", "0", "1  -2/3", 1);
    test_xgcd2("2  2 -1", "3  1 2 -3", "1  1", "2  4/7 3/7", "1  -1/7", 1);
    test_xgcd2("2  2/3 1/2", "3  1 2/3 1", "1  1", "2  12/17 -18/17", "1  9/17", 1);
    
    test_xgcd2("1  1", "0", "1  1", "1  1", "0", 1);
    test_xgcd2("2  2 3", "0", "2  2/3 1", "1  1/3", "0", 1);
    test_xgcd2("2  2 -3", "0", "2  -2/3 1", "1  -1/3", "0", 1);
    test_xgcd2("2  1 1/2", "0", "2  2 1", "1  2", "0", 1);
    test_xgcd2("2  1 3/2", "0", "2  2/3 1", "1  2/3", "0", 1);
    test_xgcd2("2  -1 -1/2", "0", "2  2 1", "1  -2", "0", 1);
    test_xgcd2("2  -1 -3/2", "0", "2  2/3 1", "1  -2/3", "0", 1);
    
    test_xgcd2("3  4 0 -1", "4  2 1 1 -1", "2  -2 1", "2  -1/3 1/3", "1  -1/3", 1);
    test_xgcd2("3  4 -4 -3", "4  2 -1 -1 -3", "2  -2/3 1", "2  -1/9 1/9", "1  -1/9", 1);
    
    test_xgcd3("0", "0", "0", "0", "0", 1);
    test_xgcd3("0", "1  1", "1  1", "0", "1  1", 1);
    test_xgcd3("0", "2  2 3", "2  2/3 1", "0", "1  1/3", 1);
    test_xgcd3("0", "2  2 -3", "2  -2/3 1", "0", "1  -1/3", 1);
    test_xgcd3("0", "2  1 1/2", "2  2 1", "0", "1  2", 1);
    test_xgcd3("0", "2  1 3/2", "2  2/3 1", "0", "1  2/3", 1);
    test_xgcd3("0", "2  -1 -1/2", "2  2 1", "0", "1  -2", 1);
    test_xgcd3("0", "2  -1 -3/2", "2  2/3 1", "0", "1  -2/3", 1);
    test_xgcd3("2  2 -1", "3  1 2 -3", "1  1", "2  4/7 3/7", "1  -1/7", 1);
    test_xgcd3("2  2/3 1/2", "3  1 2/3 1", "1  1", "2  12/17 -18/17", "1  9/17", 1);
    
    test_xgcd3("1  1", "0", "1  1", "1  1", "0", 1);
    test_xgcd3("2  2 3", "0", "2  2/3 1", "1  1/3", "0", 1);
    test_xgcd3("2  2 -3", "0", "2  -2/3 1", "1  -1/3", "0", 1);
    test_xgcd3("2  1 1/2", "0", "2  2 1", "1  2", "0", 1);
    test_xgcd3("2  1 3/2", "0", "2  2/3 1", "1  2/3", "0", 1);
    test_xgcd3("2  -1 -1/2", "0", "2  2 1", "1  -2", "0", 1);
    test_xgcd3("2  -1 -3/2", "0", "2  2/3 1", "1  -2/3", "0", 1);
    
    test_xgcd3("3  4 0 -1", "4  2 1 1 -1", "2  -2 1", "2  -1/3 1/3", "1  -1/3", 1);
    test_xgcd3("3  4 -4 -3", "4  2 -1 -1 -3", "2  -2/3 1", "2  -1/9 1/9", "1  -1/9", 1);
    
    test_xgcd4("0", "0", "0", "0", "0", 1);
    test_xgcd4("0", "1  1", "1  1", "0", "1  1", 1);
    test_xgcd4("0", "2  2 3", "2  2/3 1", "0", "1  1/3", 1);
    test_xgcd4("0", "2  2 -3", "2  -2/3 1", "0", "1  -1/3", 1);
    test_xgcd4("0", "2  1 1/2", "2  2 1", "0", "1  2", 1);
    test_xgcd4("0", "2  1 3/2", "2  2/3 1", "0", "1  2/3", 1);
    test_xgcd4("0", "2  -1 -1/2", "2  2 1", "0", "1  -2", 1);
    test_xgcd4("0", "2  -1 -3/2", "2  2/3 1", "0", "1  -2/3", 1);
    test_xgcd4("2  2 -1", "3  1 2 -3", "1  1", "2  4/7 3/7", "1  -1/7", 1);
    test_xgcd4("2  2/3 1/2", "3  1 2/3 1", "1  1", "2  12/17 -18/17", "1  9/17", 1);
    
    test_xgcd4("1  1", "0", "1  1", "1  1", "0", 1);
    test_xgcd4("2  2 3", "0", "2  2/3 1", "1  1/3", "0", 1);
    test_xgcd4("2  2 -3", "0", "2  -2/3 1", "1  -1/3", "0", 1);
    test_xgcd4("2  1 1/2", "0", "2  2 1", "1  2", "0", 1);
    test_xgcd4("2  1 3/2", "0", "2  2/3 1", "1  2/3", "0", 1);
    test_xgcd4("2  -1 -1/2", "0", "2  2 1", "1  -2", "0", 1);
    test_xgcd4("2  -1 -3/2", "0", "2  2/3 1", "1  -2/3", "0", 1);
    
    test_xgcd4("3  4 0 -1", "4  2 1 1 -1", "2  -2 1", "2  -1/3 1/3", "1  -1/3", 1);
    test_xgcd4("3  4 -4 -3", "4  2 -1 -1 -3", "2  -2/3 1", "2  -1/9 1/9", "1  -1/9", 1);
    
    test_xgcd5("0", "0", "0", "0", "0", 1);
    test_xgcd5("0", "1  1", "1  1", "0", "1  1", 1);
    test_xgcd5("0", "2  2 3", "2  2/3 1", "0", "1  1/3", 1);
    test_xgcd5("0", "2  2 -3", "2  -2/3 1", "0", "1  -1/3", 1);
    test_xgcd5("0", "2  1 1/2", "2  2 1", "0", "1  2", 1);
    test_xgcd5("0", "2  1 3/2", "2  2/3 1", "0", "1  2/3", 1);
    test_xgcd5("0", "2  -1 -1/2", "2  2 1", "0", "1  -2", 1);
    test_xgcd5("0", "2  -1 -3/2", "2  2/3 1", "0", "1  -2/3", 1);
    test_xgcd5("2  2 -1", "3  1 2 -3", "1  1", "2  4/7 3/7", "1  -1/7", 1);
    test_xgcd5("2  2/3 1/2", "3  1 2/3 1", "1  1", "2  12/17 -18/17", "1  9/17", 1);
    
    test_xgcd5("1  1", "0", "1  1", "1  1", "0", 1);
    test_xgcd5("2  2 3", "0", "2  2/3 1", "1  1/3", "0", 1);
    test_xgcd5("2  2 -3", "0", "2  -2/3 1", "1  -1/3", "0", 1);
    test_xgcd5("2  1 1/2", "0", "2  2 1", "1  2", "0", 1);
    test_xgcd5("2  1 3/2", "0", "2  2/3 1", "1  2/3", "0", 1);
    test_xgcd5("2  -1 -1/2", "0", "2  2 1", "1  -2", "0", 1);
    test_xgcd5("2  -1 -3/2", "0", "2  2/3 1", "1  -2/3", "0", 1);
    
    test_xgcd5("3  4 0 -1", "4  2 1 1 -1", "2  -2 1", "2  -1/3 1/3", "1  -1/3", 1);
    test_xgcd5("3  4 -4 -3", "4  2 -1 -1 -3", "2  -2/3 1", "2  -1/9 1/9", "1  -1/9", 1);
    
    test_xgcd6("0", "0", "0", "0", "0", 1);
    test_xgcd6("0", "1  1", "1  1", "0", "1  1", 1);
    test_xgcd6("0", "2  2 3", "2  2/3 1", "0", "1  1/3", 1);
    test_xgcd6("0", "2  2 -3", "2  -2/3 1", "0", "1  -1/3", 1);
    test_xgcd6("0", "2  1 1/2", "2  2 1", "0", "1  2", 1);
    test_xgcd6("0", "2  1 3/2", "2  2/3 1", "0", "1  2/3", 1);
    test_xgcd6("0", "2  -1 -1/2", "2  2 1", "0", "1  -2", 1);
    test_xgcd6("0", "2  -1 -3/2", "2  2/3 1", "0", "1  -2/3", 1);
    test_xgcd6("2  2 -1", "3  1 2 -3", "1  1", "2  4/7 3/7", "1  -1/7", 1);
    test_xgcd6("2  2/3 1/2", "3  1 2/3 1", "1  1", "2  12/17 -18/17", "1  9/17", 1);
    
    test_xgcd6("1  1", "0", "1  1", "1  1", "0", 1);
    test_xgcd6("2  2 3", "0", "2  2/3 1", "1  1/3", "0", 1);
    test_xgcd6("2  2 -3", "0", "2  -2/3 1", "1  -1/3", "0", 1);
    test_xgcd6("2  1 1/2", "0", "2  2 1", "1  2", "0", 1);
    test_xgcd6("2  1 3/2", "0", "2  2/3 1", "1  2/3", "0", 1);
    test_xgcd6("2  -1 -1/2", "0", "2  2 1", "1  -2", "0", 1);
    test_xgcd6("2  -1 -3/2", "0", "2  2/3 1", "1  -2/3", "0", 1);
    
    test_xgcd6("3  4 0 -1", "4  2 1 1 -1", "2  -2 1", "2  -1/3 1/3", "1  -1/3", 1);
    test_xgcd6("3  4 -4 -3", "4  2 -1 -1 -3", "2  -2/3 1", "2  -1/9 1/9", "1  -1/9", 1);
    
    test_xgcd7("0", "0", "0", "0", "0", 1);
    test_xgcd7("0", "1  1", "1  1", "0", "1  1", 1);
    test_xgcd7("0", "2  2 3", "2  2/3 1", "0", "1  1/3", 1);
    test_xgcd7("0", "2  2 -3", "2  -2/3 1", "0", "1  -1/3", 1);
    test_xgcd7("0", "2  1 1/2", "2  2 1", "0", "1  2", 1);
    test_xgcd7("0", "2  1 3/2", "2  2/3 1", "0", "1  2/3", 1);
    test_xgcd7("0", "2  -1 -1/2", "2  2 1", "0", "1  -2", 1);
    test_xgcd7("0", "2  -1 -3/2", "2  2/3 1", "0", "1  -2/3", 1);
    test_xgcd7("2  2 -1", "3  1 2 -3", "1  1", "2  4/7 3/7", "1  -1/7", 1);
    test_xgcd7("2  2/3 1/2", "3  1 2/3 1", "1  1", "2  12/17 -18/17", "1  9/17", 1);
    
    test_xgcd7("1  1", "0", "1  1", "1  1", "0", 1);
    test_xgcd7("2  2 3", "0", "2  2/3 1", "1  1/3", "0", 1);
    test_xgcd7("2  2 -3", "0", "2  -2/3 1", "1  -1/3", "0", 1);
    test_xgcd7("2  1 1/2", "0", "2  2 1", "1  2", "0", 1);
    test_xgcd7("2  1 3/2", "0", "2  2/3 1", "1  2/3", "0", 1);
    test_xgcd7("2  -1 -1/2", "0", "2  2 1", "1  -2", "0", 1);
    test_xgcd7("2  -1 -3/2", "0", "2  2/3 1", "1  -2/3", "0", 1);
    
    test_xgcd7("3  4 0 -1", "4  2 1 1 -1", "2  -2 1", "2  -1/3 1/3", "1  -1/3", 1);
    test_xgcd7("3  4 -4 -3", "4  2 -1 -1 -3", "2  -2/3 1", "2  -1/9 1/9", "1  -1/9", 1);
    
    test_xgcd8("0", "0", "0", "0", "0", 1);
    test_xgcd8("0", "1  1", "1  1", "0", "1  1", 1);
    test_xgcd8("0", "2  2 3", "2  2/3 1", "0", "1  1/3", 1);
    test_xgcd8("0", "2  2 -3", "2  -2/3 1", "0", "1  -1/3", 1);
    test_xgcd8("0", "2  1 1/2", "2  2 1", "0", "1  2", 1);
    test_xgcd8("0", "2  1 3/2", "2  2/3 1", "0", "1  2/3", 1);
    test_xgcd8("0", "2  -1 -1/2", "2  2 1", "0", "1  -2", 1);
    test_xgcd8("0", "2  -1 -3/2", "2  2/3 1", "0", "1  -2/3", 1);
    test_xgcd8("2  2 -1", "3  1 2 -3", "1  1", "2  4/7 3/7", "1  -1/7", 1);
    test_xgcd8("2  2/3 1/2", "3  1 2/3 1", "1  1", "2  12/17 -18/17", "1  9/17", 1);
    
    test_xgcd8("1  1", "0", "1  1", "1  1", "0", 1);
    test_xgcd8("2  2 3", "0", "2  2/3 1", "1  1/3", "0", 1);
    test_xgcd8("2  2 -3", "0", "2  -2/3 1", "1  -1/3", "0", 1);
    test_xgcd8("2  1 1/2", "0", "2  2 1", "1  2", "0", 1);
    test_xgcd8("2  1 3/2", "0", "2  2/3 1", "1  2/3", "0", 1);
    test_xgcd8("2  -1 -1/2", "0", "2  2 1", "1  -2", "0", 1);
    test_xgcd8("2  -1 -3/2", "0", "2  2/3 1", "1  -2/3", "0", 1);
    
    test_xgcd8("3  4 0 -1", "4  2 1 1 -1", "2  -2 1", "2  -1/3 1/3", "1  -1/3", 1);
    test_xgcd8("3  4 -4 -3", "4  2 -1 -1 -3", "2  -2/3 1", "2  -1/9 1/9", "1  -1/9", 1);
    
    test_xgcd9("0", "0", "0", "0", "0", 1);
    test_xgcd9("0", "1  1", "1  1", "0", "1  1", 1);
    test_xgcd9("0", "2  2 3", "2  2/3 1", "0", "1  1/3", 1);
    test_xgcd9("0", "2  2 -3", "2  -2/3 1", "0", "1  -1/3", 1);
    test_xgcd9("0", "2  1 1/2", "2  2 1", "0", "1  2", 1);
    test_xgcd9("0", "2  1 3/2", "2  2/3 1", "0", "1  2/3", 1);
    test_xgcd9("0", "2  -1 -1/2", "2  2 1", "0", "1  -2", 1);
    test_xgcd9("0", "2  -1 -3/2", "2  2/3 1", "0", "1  -2/3", 1);
    test_xgcd9("2  2 -1", "3  1 2 -3", "1  1", "2  4/7 3/7", "1  -1/7", 1);
    test_xgcd9("2  2/3 1/2", "3  1 2/3 1", "1  1", "2  12/17 -18/17", "1  9/17", 1);
    
    test_xgcd9("1  1", "0", "1  1", "1  1", "0", 1);
    test_xgcd9("2  2 3", "0", "2  2/3 1", "1  1/3", "0", 1);
    test_xgcd9("2  2 -3", "0", "2  -2/3 1", "1  -1/3", "0", 1);
    test_xgcd9("2  1 1/2", "0", "2  2 1", "1  2", "0", 1);
    test_xgcd9("2  1 3/2", "0", "2  2/3 1", "1  2/3", "0", 1);
    test_xgcd9("2  -1 -1/2", "0", "2  2 1", "1  -2", "0", 1);
    test_xgcd9("2  -1 -3/2", "0", "2  2/3 1", "1  -2/3", "0", 1);
    
    test_xgcd9("3  4 0 -1", "4  2 1 1 -1", "2  -2 1", "2  -1/3 1/3", "1  -1/3", 1);
    test_xgcd9("3  4 -4 -3", "4  2 -1 -1 -3", "2  -2/3 1", "2  -1/9 1/9", "1  -1/9", 1);
    
    test_xgcd10("0", "0", "0", "0", "0", 1);
    test_xgcd10("0", "1  1", "1  1", "0", "1  1", 1);
    test_xgcd10("0", "2  2 3", "2  2/3 1", "0", "1  1/3", 1);
    test_xgcd10("0", "2  2 -3", "2  -2/3 1", "0", "1  -1/3", 1);
    test_xgcd10("0", "2  1 1/2", "2  2 1", "0", "1  2", 1);
    test_xgcd10("0", "2  1 3/2", "2  2/3 1", "0", "1  2/3", 1);
    test_xgcd10("0", "2  -1 -1/2", "2  2 1", "0", "1  -2", 1);
    test_xgcd10("0", "2  -1 -3/2", "2  2/3 1", "0", "1  -2/3", 1);
    test_xgcd10("2  2 -1", "3  1 2 -3", "1  1", "2  4/7 3/7", "1  -1/7", 1);
    test_xgcd10("2  2/3 1/2", "3  1 2/3 1", "1  1", "2  12/17 -18/17", "1  9/17", 1);
    
    test_xgcd10("1  1", "0", "1  1", "1  1", "0", 1);
    test_xgcd10("2  2 3", "0", "2  2/3 1", "1  1/3", "0", 1);
    test_xgcd10("2  2 -3", "0", "2  -2/3 1", "1  -1/3", "0", 1);
    test_xgcd10("2  1 1/2", "0", "2  2 1", "1  2", "0", 1);
    test_xgcd10("2  1 3/2", "0", "2  2/3 1", "1  2/3", "0", 1);
    test_xgcd10("2  -1 -1/2", "0", "2  2 1", "1  -2", "0", 1);
    test_xgcd10("2  -1 -3/2", "0", "2  2/3 1", "1  -2/3", "0", 1);
    
    test_xgcd10("3  4 0 -1", "4  2 1 1 -1", "2  -2 1", "2  -1/3 1/3", "1  -1/3", 1);
    test_xgcd10("3  4 -4 -3", "4  2 -1 -1 -3", "2  -2/3 1", "2  -1/9 1/9", "1  -1/9", 1);
    
    test_binop1(lcm, "0", "0", "0", 1);
    test_binop1(lcm, "0", "2  1/2 1", "0", 1);
    test_binop1(lcm, "3  -1/2 2 3/7", "0", "0", 1);
    test_binop1(lcm, "2  3 -1", "2  2 -1", "3  6 -5 1", 1);
    test_binop1(lcm, "2  3 -1", "3  3 2 -1", "3  -3 -2 1", 1);
    test_binop1(lcm, "2  2 3", "3  2 5 3", "3  2/3 5/3 1", 1);
    
    test_binop3(lcm, "0", "0", "0", 1);
    test_binop3(lcm, "0", "2  1/2 1", "0", 1);
    test_binop3(lcm, "3  -1/2 2 3/7", "0", "0", 1);
    test_binop3(lcm, "2  3 -1", "2  2 -1", "3  6 -5 1", 1);
    test_binop3(lcm, "2  3 -1", "3  3 2 -1", "3  -3 -2 1", 1);
    test_binop3(lcm, "2  2 3", "3  2 5 3", "3  2/3 5/3 1", 1);
    
    test_binop4(lcm, "0", "0", "0", 1);
    test_binop4(lcm, "0", "2  1/2 1", "0", 1);
    test_binop4(lcm, "3  -1/2 2 3/7", "0", "0", 1);
    test_binop4(lcm, "2  3 -1", "2  2 -1", "3  6 -5 1", 1);
    test_binop4(lcm, "2  3 -1", "3  3 2 -1", "3  -3 -2 1", 1);
    test_binop4(lcm, "2  2 3", "3  2 5 3", "3  2/3 5/3 1", 1);
    
    //-----------------------------------------------------------------------//
    // Derivative                                                            //
    //-----------------------------------------------------------------------//
    
    test_derivative1("0", "0", 1);
    test_derivative1("1  2", "0", 1);
    test_derivative1("1  2/3", "0", 1);
    test_derivative1("2  1/2 2", "1  2", 1);
    
    test_derivative2("0", "0", 1);
    test_derivative2("1  2", "0", 1);
    test_derivative2("1  2/3", "0", 1);
    test_derivative2("2  1/2 2", "1  2", 1);
    
    //-----------------------------------------------------------------------//
    // Evaluation                                                            //
    //-----------------------------------------------------------------------//
    
    mpz_set_si(mpz1, 2);
    test_evaluate_mpz("0", mpz1, "0", 1);
    test_evaluate_mpz("1  1/2", mpz1, "1/2", 1);
    test_evaluate_mpz("2  2 3", mpz1, "8", 1);
    test_evaluate_mpz("2  3 1/2", mpz1, "4", 1);
    test_evaluate_mpz("2  3/2 1/2", mpz1, "5/2", 1);
    
    mpq_set_si(mpq1, 0, 1);
    mpq_set_si(mpq2, 1, 2);
    mpq_set_si(mpq3, -2, 1);
    
    test_evaluate_mpq1("0", mpq1, "0", 1);
    test_evaluate_mpq1("0", mpq2, "0", 1);
    test_evaluate_mpq1("0", mpq3, "0", 1);
    test_evaluate_mpq1("1  2", mpq1, "2", 1);
    test_evaluate_mpq1("1  2", mpq2, "2", 1);
    test_evaluate_mpq1("1  2", mpq3, "2", 1);
    test_evaluate_mpq1("1  2/3", mpq1, "2/3", 1);
    test_evaluate_mpq1("1  2/3", mpq2, "2/3", 1);
    test_evaluate_mpq1("1  2/3", mpq3, "2/3", 1);
    test_evaluate_mpq1("3  2 3 -4", mpq1, "2", 1);
    test_evaluate_mpq1("3  2 3 -4", mpq2, "5/2", 1);
    test_evaluate_mpq1("3  2 3 -4", mpq3, "-20", 1);
    test_evaluate_mpq1("4  2/3 2 -3 1/5", mpq1, "2/3", 1);
    test_evaluate_mpq1("4  2/3 2 -3 1/5", mpq2, "113/120", 1);
    test_evaluate_mpq1("4  2/3 2 -3 1/5", mpq3, "-254/15", 1);
    
    test_evaluate_mpq2("0", mpq1, "0", 1);
    test_evaluate_mpq2("0", mpq2, "0", 1);
    test_evaluate_mpq2("0", mpq3, "0", 1);
    test_evaluate_mpq2("1  2", mpq1, "2", 1);
    test_evaluate_mpq2("1  2", mpq2, "2", 1);
    test_evaluate_mpq2("1  2", mpq3, "2", 1);
    test_evaluate_mpq2("1  2/3", mpq1, "2/3", 1);
    test_evaluate_mpq2("1  2/3", mpq2, "2/3", 1);
    test_evaluate_mpq2("1  2/3", mpq3, "2/3", 1);
    test_evaluate_mpq2("3  2 3 -4", mpq1, "2", 1);
    test_evaluate_mpq2("3  2 3 -4", mpq2, "5/2", 1);
    test_evaluate_mpq2("3  2 3 -4", mpq3, "-20", 1);
    test_evaluate_mpq2("4  2/3 2 -3 1/5", mpq1, "2/3", 1);
    test_evaluate_mpq2("4  2/3 2 -3 1/5", mpq2, "113/120", 1);
    test_evaluate_mpq2("4  2/3 2 -3 1/5", mpq3, "-254/15", 1);
    
    //-----------------------------------------------------------------------//
    // Gaussian content                                                      //
    //-----------------------------------------------------------------------//
    
    test_content("0", "0", 1);
    test_content("1  2", "2", 1);
    test_content("1  1/2", "1/2", 1);
    test_content("3  2/3 1 2", "1/3", 1);
    test_content("3  2/3 2 4", "2/3", 1);
    
    test_primitive_part1("0", "0", 1);
    test_primitive_part1("1  2", "1  1", 1);
    test_primitive_part1("1  -2/17", "1  1", 1);
    test_primitive_part1("2  2 -1/3", "2  -6 1", 1);
    test_primitive_part1("3  2/3 2 4", "3  1 3 6", 1);
    
    test_primitive_part2("0", "0", 1);
    test_primitive_part2("1  2", "1  1", 1);
    test_primitive_part2("1  -2/17", "1  1", 1);
    test_primitive_part2("2  2 -1/3", "2  -6 1", 1);
    test_primitive_part2("3  2/3 2 4", "3  1 3 6", 1);
    
    test_is_monic("0", 0, 1);
    test_is_monic("1  2", 0, 1);
    test_is_monic("1  1", 1, 1);
    test_is_monic("1  -1", 0, 1);
    test_is_monic("3  1 2/3 1/2", 0, 1);
    test_is_monic("3  1 2/3 1", 1, 1);
    
    //-----------------------------------------------------------------------//
    // Resultant                                                             //
    //-----------------------------------------------------------------------//
    
    test_resultant("0", "0", "0", 1);
    test_resultant("0", "1  1", "0", 1);
    test_resultant("0", "2  1 1", "0", 1);
    test_resultant("1  1", "1  -2/3", "1", 1);
    test_resultant("1  1", "2  1 1", "1", 1);
    test_resultant("1  1/2", "2  1 1", "1/2", 1);
    test_resultant("1  -3/2", "2  1 1", "-3/2", 1);
    test_resultant("1  1", "2  3 2/5", "1", 1);
    test_resultant("1  1/2", "2  3 2/5", "1/2", 1);
    test_resultant("1  -3/2", "2  3 2/5", "-3/2", 1);
    test_resultant("1  2/3", "3  2 3 1", "4/9", 1);
    test_resultant("1  2/3", "3  2 3 1/5", "4/9", 1);
    test_resultant("1  2", "3  2 3 1", "4", 1);
    test_resultant("1  2", "3  2 3 1/5", "4", 1);
    test_resultant("3  1 0 -1", "2  1 1", "0", 1);
    test_resultant("3  1/3 0 -1/3", "2  1 1", "0", 1);
    test_resultant("3  1/2 0 -1/3", "2  1 1", "1/6", 1);
    test_resultant("2  1 1", "1  1", "1", 1);
    test_resultant("3  1 2 3", "3  2 4 8", "4", 1);
    test_resultant("2  2 2", "3  3 6 9", "24", 1);
    test_resultant("3  1 2 3", "3  2/17 4 8", "12548/289", 1);
    test_resultant("2  2 2", "3  3 6/7 9", "312/7", 1);
    test_resultant("2  2/3 2", "3  3 6/7 9", "104/7", 1);
    test_resultant("2  1 2", "1  2", "2", 1);
    
    test_discriminant("0", "0", 1);
    test_discriminant("1  1", "0", 1);
    test_discriminant("1  -2/3", "0", 1);
    test_discriminant("2  1 1", "1", 1);
    test_discriminant("2  1/2 1", "1", 1);
    test_discriminant("2  1/2 -3", "1", 1);
    test_discriminant("3  -1 2 3", "16", 1);
    test_discriminant("3  -5 0 1", "20", 1);
    test_discriminant("3  -1 2/9 1/3", "112/81", 1);
    test_discriminant("3  -2/5 0 9", "72/5", 1);
    
    //-----------------------------------------------------------------------//
    // Composition                                                           //
    //-----------------------------------------------------------------------//
    
    test_binop1(compose, "0", "0", "0", 1);
    test_binop1(compose, "0", "1  1/2", "0", 1);
    test_binop1(compose, "1  1/2", "0", "1  1/2", 1);
    test_binop1(compose, "1  1/2", "1  -2/3", "1  1/2", 1);
    test_binop1(compose, "1  1/2", "2  3/2 -2/5", "1  1/2", 1);
    test_binop1(compose, "2  1 2", "0", "1  1", 1);
    test_binop1(compose, "2  1 2", "1  2", "1  5", 1);
    test_binop1(compose, "2  1 2", "2  2/3 1/2", "2  7/3 1", 1);
    test_binop1(compose, "2  4/7 -1/3", "0", "1  4/7", 1);
    test_binop1(compose, "2  4/7 -1/3", "1  2/3", "1  22/63", 1);
    test_binop1(compose, "2  4/7 -1/3", "2  1 1", "2  5/21 -1/3", 1);
    test_binop1(compose, "2  4/7 -1/3", "2  2/3 5/7", "2  22/63 -5/21", 1);
    
    test_binop3(compose, "0", "0", "0", 1);
    test_binop3(compose, "0", "1  1/2", "0", 1);
    test_binop3(compose, "1  1/2", "0", "1  1/2", 1);
    test_binop3(compose, "1  1/2", "1  -2/3", "1  1/2", 1);
    test_binop3(compose, "1  1/2", "2  3/2 -2/5", "1  1/2", 1);
    test_binop3(compose, "2  1 2", "0", "1  1", 1);
    test_binop3(compose, "2  1 2", "1  2", "1  5", 1);
    test_binop3(compose, "2  1 2", "2  2/3 1/2", "2  7/3 1", 1);
    test_binop3(compose, "2  4/7 -1/3", "0", "1  4/7", 1);
    test_binop3(compose, "2  4/7 -1/3", "1  2/3", "1  22/63", 1);
    test_binop3(compose, "2  4/7 -1/3", "2  1 1", "2  5/21 -1/3", 1);
    test_binop3(compose, "2  4/7 -1/3", "2  2/3 5/7", "2  22/63 -5/21", 1);
    
    test_binop4(compose, "0", "0", "0", 1);
    test_binop4(compose, "0", "1  1/2", "0", 1);
    test_binop4(compose, "1  1/2", "0", "1  1/2", 1);
    test_binop4(compose, "1  1/2", "1  -2/3", "1  1/2", 1);
    test_binop4(compose, "1  1/2", "2  3/2 -2/5", "1  1/2", 1);
    test_binop4(compose, "2  1 2", "0", "1  1", 1);
    test_binop4(compose, "2  1 2", "1  2", "1  5", 1);
    test_binop4(compose, "2  1 2", "2  2/3 1/2", "2  7/3 1", 1);
    test_binop4(compose, "2  4/7 -1/3", "0", "1  4/7", 1);
    test_binop4(compose, "2  4/7 -1/3", "1  2/3", "1  22/63", 1);
    test_binop4(compose, "2  4/7 -1/3", "2  1 1", "2  5/21 -1/3", 1);
    test_binop4(compose, "2  4/7 -1/3", "2  2/3 5/7", "2  22/63 -5/21", 1);
    
    mpq_set_si(mpq1, 0, 1);
    mpq_set_si(mpq2, 2, 1);
    mpq_set_si(mpq3, 2, 3);
    
    test_rescale1("0", mpq1, "0", 1);
    test_rescale1("0", mpq2, "0", 1);
    test_rescale1("0", mpq3, "0", 1);
    test_rescale1("1  -3/2", mpq1, "1  -3/2", 1);
    test_rescale1("1  -3/2", mpq2, "1  -3/2", 1);
    test_rescale1("1  -3/2", mpq3, "1  -3/2", 1);
    test_rescale1("2  2 1", mpq1, "1  2", 1);
    test_rescale1("2  2 1", mpq2, "2  2 2", 1);
    test_rescale1("2  2 1", mpq3, "2  2 2/3", 1);
    test_rescale1("2  2/3 4/7", mpq1, "1  2/3", 1);
    test_rescale1("2  2/3 4/7", mpq2, "2  2/3 8/7", 1);
    test_rescale1("2  2/3 4/7", mpq3, "2  2/3 8/21", 1);
    test_rescale1("3  2 1/3 -1", mpq1, "1  2", 1);
    test_rescale1("3  2 1/3 -1", mpq2, "3  2 2/3 -4", 1);
    test_rescale1("3  2 1/3 -1", mpq3, "3  2 2/9 -4/9", 1);
    
    test_rescale2("0", mpq1, "0", 1);
    test_rescale2("0", mpq2, "0", 1);
    test_rescale2("0", mpq3, "0", 1);
    test_rescale2("1  -3/2", mpq1, "1  -3/2", 1);
    test_rescale2("1  -3/2", mpq2, "1  -3/2", 1);
    test_rescale2("1  -3/2", mpq3, "1  -3/2", 1);
    test_rescale2("2  2 1", mpq1, "1  2", 1);
    test_rescale2("2  2 1", mpq2, "2  2 2", 1);
    test_rescale2("2  2 1", mpq3, "2  2 2/3", 1);
    test_rescale2("2  2/3 4/7", mpq1, "1  2/3", 1);
    test_rescale2("2  2/3 4/7", mpq2, "2  2/3 8/7", 1);
    test_rescale2("2  2/3 4/7", mpq3, "2  2/3 8/21", 1);
    test_rescale2("3  2 1/3 -1", mpq1, "1  2", 1);
    test_rescale2("3  2 1/3 -1", mpq2, "3  2 2/3 -4", 1);
    test_rescale2("3  2 1/3 -1", mpq3, "3  2 2/9 -4/9", 1);
    
    //-----------------------------------------------------------------------//
    // Square-free                                                           //
    //-----------------------------------------------------------------------//
    
    test_is_squarefree("0", 1, 1);
    test_is_squarefree("1  1/3", 1, 1);
    test_is_squarefree("3  1/2 1/2 -1", 1, 1);
    test_is_squarefree("3  16/25 16/5 4", 0, 1);
    
    //-----------------------------------------------------------------------//
    // Subpolynomials                                                        //
    //-----------------------------------------------------------------------//
    
    test_getslice1("4  1/2 3 1 3", 0, 4, "4  1/2 3 1 3", 1);
    test_getslice1("4  1/2 3 1 3", 3, 2, "0", 1);
    test_getslice1("4  1/2 3 1 3", 2, 5, "4  0 0 1 3", 1);
    test_getslice1("3  1 2 3", 0, 2, "2  1 2", 1);
    
    test_getslice2("4  1/2 3 1 3", 0, 4, "4  1/2 3 1 3", 1);
    test_getslice2("4  1/2 3 1 3", 3, 2, "0", 1);
    test_getslice2("4  1/2 3 1 3", 2, 5, "4  0 0 1 3", 1);
    test_getslice2("3  1 2 3", 0, 2, "2  1 2", 1);
    
    test_left_shift1("0", 0, "0", 1);
    test_left_shift1("0", 2, "0", 1);
    test_left_shift1("1  1", 0, "1  1", 1);
    test_left_shift1("1  1", 1, "2  0 1", 1);
    
    test_left_shift2("0", 0, "0", 1);
    test_left_shift2("0", 2, "0", 1);
    test_left_shift2("1  1", 0, "1  1", 1);
    test_left_shift2("1  1", 1, "2  0 1", 1);
    
    test_right_shift1("0", 1, "0", 1);
    test_right_shift1("0", 0, "0", 1);
    test_right_shift1("1  1", 1, "0", 1);
    test_right_shift1("1  1", 0, "1  1", 1);
    
    test_right_shift2("0", 1, "0", 1);
    test_right_shift2("0", 0, "0", 1);
    test_right_shift2("1  1", 1, "0", 1);
    test_right_shift2("1  1", 0, "1  1", 1);

    test_truncate1("0", 0, "0", 1);
    test_truncate1("0", 1, "0", 1);
    test_truncate1("1  2", 0, "0", 1);
    test_truncate1("1  2", 1, "1  2", 1);
    test_truncate1("2  1 2/3", 2, "2  1 2/3", 1);
    test_truncate1("2  1 2/3", 1, "1  1", 1);
    
    test_truncate2("0", 0, "0", 1);
    test_truncate2("0", 1, "0", 1);
    test_truncate2("1  2", 0, "0", 1);
    test_truncate2("1  2", 1, "1  2", 1);
    test_truncate2("2  1 2/3", 2, "2  1 2/3", 1);
    test_truncate2("2  1 2/3", 1, "1  1", 1);
    
    test_reverse1("0", "0", 1);
    test_reverse1("1  -1/2", "1  -1/2", 1);
    test_reverse1("2  -1 2", "2  2 -1", 1);
    test_reverse1("3  -2/3 2 1/2", "3  1/2 2 -2/3", 1);
    
    test_reverse2("0", "0", 1);
    test_reverse2("1  -1/2", "1  -1/2", 1);
    test_reverse2("2  -1 2", "2  2 -1", 1);
    test_reverse2("3  -2/3 2 1/2", "3  1/2 2 -2/3", 1);
    
    //-----------------------------------------------------------------------//
    // String conversion                                                     //
    //-----------------------------------------------------------------------//
    
    test_to_string_pretty("0", "t", "0", 1);
    test_to_string_pretty("1  2/3", "t", "2/3", 1);
    test_to_string_pretty("1  -1", "t", "-1", 1);
    test_to_string_pretty("2  1 2", "t", "2*t + 1", 1);
    test_to_string_pretty("2  1/2 3/5", "t", "3/5*t + 1/2", 1);
    test_to_string_pretty("3  1 -2/3 3", "t", "3*t^2 - 2/3*t + 1", 1);
    test_to_string_pretty("4  1 0 0 2/3", "x", "2/3*x^3 + 1", 1);
    test_to_string_pretty("6  -1/2 2 3/2 3/7 -2 -1", "Var", "-Var^5 - 2*Var^4 + 3/7*Var^3 + 3/2*Var^2 + 2*Var - 1/2", 1);
    
    //-----------------------------------------------------------------------//
    
    fmpz_clear(fmpz1);
    fmpz_clear(fmpz2);
    fmpz_clear(fmpz3);
    mpz_clear(mpz1);
    mpz_clear(mpz2);
    mpz_clear(mpz3);
    mpq_clear(mpq1);
    mpq_clear(mpq2);
    mpq_clear(mpq3);
    
    flint_stack_cleanup();
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

