///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

/**
 * \file     fmpq_poly-test.c
 * \brief    Test file for code coverage in fmpq_poly.c and fmpq_poly.h
 * \author   Sebastian Pancratz
 * \date     July 2010
 * \version  0.1.7
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <gmp.h>

#include "flint.h"
#include "mpz_poly.h"
#include "fmpz_poly.h"
#include "fmpq_poly.h"

void fmpz_randtest(fmpz_t * f, gmp_randstate_t R, ulong bits)
{
    mpz_t g;
    mpz_init(g);
    mpz_rrandomb(g, R, bits);
    if (rand() % 2)
        mpz_clrbit(g, bits - 1);
    if (rand() % 2)
        mpz_neg(g, g);
    *f = fmpz_realloc(*f, mpz_size(g));
    mpz_to_fmpz(*f, g);
    mpz_clear(g);
}

void fmpz_randtest_not_zero(fmpz_t * f, gmp_randstate_t R, ulong bits)
{
    mpz_t g;
    mpz_init(g);
    mpz_rrandomb(g, R, bits);
    if (z_randint(2))
        mpz_clrbit(g, bits - 1);
    if (rand() % 2)
        mpz_neg(g, g);
    
    if (mpz_sgn(g) == 0)
        fmpz_set_si(*f, -123);
    else
    {
        *f = fmpz_realloc(*f, mpz_size(g));
        mpz_to_fmpz(*f, g);
    }
    
    mpz_clear(g);
}

void fmpz_poly_randtest(fmpz_poly_t poly, gmp_randstate_t R, ulong len, ulong bits)
{
    long i;
    mpz_t temp;
    mpz_poly_t mpoly;

    if (len == 0 || bits == 0)
    {
        fmpz_poly_zero(poly);
        return;
    }

    mpz_init(temp);
    mpz_poly_init(mpoly);
    mpz_poly_zero(mpoly);

    fmpz_poly_fit_length(poly, len);
   
    for (i = 0; i < len; i++)
    {
        mpz_rrandomb(temp, R, bits);
        if (z_randint(2))
            mpz_clrbit(temp, bits - 1);
        if (rand() % 2)
            mpz_neg(temp,temp);
        mpz_poly_set_coeff(mpoly, i, temp);
    }

    mpz_poly_to_fmpz_poly(poly, mpoly);

    mpz_clear(temp);
    mpz_poly_clear(mpoly);
} 

void fmpz_poly_randtest_not_zero(fmpz_poly_t poly, gmp_randstate_t R, ulong len, ulong bits)
{
    if (len == 0ul)
        len = 1ul;
    if (bits == 0ul)
        bits = 23ul;
    fmpz_poly_randtest(poly, R, len, bits);
    if (poly->length == 0ul)
        fmpz_poly_set_coeff_si(poly, 0, 1);
}

void fmpq_poly_randtest(fmpq_poly_t poly, gmp_randstate_t R, ulong len, ulong bits)
{
    fmpz_poly_randtest(poly->num, R, len, bits);
    fmpz_randtest_not_zero(&(poly->den), R, bits);
    fmpq_poly_canonicalize(poly, NULL);
} 

void fmpq_poly_randtest_not_zero(fmpq_poly_t poly, gmp_randstate_t R, ulong len, ulong bits)
{
    fmpz_poly_randtest_not_zero(poly->num, R, len, bits);
    fmpz_randtest_not_zero(&(poly->den), R, bits);
    fmpq_poly_canonicalize(poly, NULL);
} 

/* A very small example...
 * 
int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    fmpq_poly_t f;
    char * str;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, 0ul);
    
    srand(time(NULL));

    fmpq_poly_init(f);
    fmpq_poly_randtest(f, R, 10, 10);
    
    str = fmpq_poly_to_string_pretty(f, "t");
    printf(" f = %s\n", str);
    free(str);
    
    fmpq_poly_clear(f);
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}
*/

