///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("COMPOSE..."); fflush(stdout);
    
    /* Check aliasing for f and h in h = f(g) */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, h;
        ulong len1 = 50, len2 = 15, bits = 75;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(h);
        
        fmpq_poly_randtest(f, R, rand() % len1, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len2, rand() % bits);
        
        fmpq_poly_compose(h, f, g);
        fmpq_poly_compose(f, f, g);
        
        result = fmpq_poly_equal(f, h);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(f);
            str2 = fmpq_poly_to_string(h);
            printf("FAIL\n");
            printf("  f = %s\n", str1);
            printf("  h = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(h);
    }
    
    /* Check f(g(a)) == f(g)(a) */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, h;
        fmpz_t anum, aden;
        mpq_t a, b;
        ulong len1 = 50, len2 = 15, bits = 75;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(h);
        anum = fmpz_init(bits / FLINT_BITS + 2);
        aden = fmpz_init(bits / FLINT_BITS + 2);
        mpq_init(a);
        mpq_init(b);
        
        fmpq_poly_randtest(f, R, rand() % len1, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len2, rand() % bits);
        fmpz_randtest(&anum, R, rand() % bits);
        fmpz_randtest_not_zero(&aden, R, rand() % bits);
        fmpz_to_mpz(mpq_numref(a), anum);
        fmpz_to_mpz(mpq_denref(a), aden);
        mpq_canonicalize(a);
        
        fmpq_poly_compose(h, f, g);
        fmpq_poly_evaluate_mpq(b, h, a);
        fmpq_poly_evaluate_mpq(a, g, a);
        fmpq_poly_evaluate_mpq(a, f, a);
        
        result = mpq_equal(a, b);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(f);
            str2 = fmpq_poly_to_string(h);
            printf("FAIL\n");
            printf("  f = %s\n", str1);
            printf("  h = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(h);
        fmpz_clear(anum);
        fmpz_clear(aden);
        mpq_clear(a);
        mpq_clear(b);
    }
    
    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

