///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);

    printf("DERIVATIVE..."); fflush(stdout);
    
    /* Check aliasing */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        
        fmpq_poly_derivative(g, f);
        fmpq_poly_derivative(f, f);
        
        result = fmpq_poly_equal(f, g);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(f);
            strg = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
    }

    /* d(f) == 0 for constant f */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f;
        ulong len = 1, bits = 100;
        
        fmpq_poly_init(f);
        
        fmpq_poly_randtest(f, R, len, rand() % bits);
        
        fmpq_poly_derivative(f, f);
        
        result = fmpq_poly_is_zero(f);
        if (!result)
        {
            char * strf;
            strf = fmpq_poly_to_string(f);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            free(strf);
            abort();
        }
        
        fmpq_poly_clear(f);
    }

    /* d(f + g) == df + dg */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, der;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(der);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        
        fmpq_poly_add(der, f, g);
        fmpq_poly_derivative(der, der);
        fmpq_poly_derivative(f, f);
        fmpq_poly_derivative(g, g);
        fmpq_poly_add(f, f, g);
        
        result = fmpq_poly_equal(der, f);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(der);
            strg = fmpq_poly_to_string(f);
            printf("FAIL\n");
            printf("  lhs = %s\n", strf);
            printf("  rhs = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(der);
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
    }

    /* d(a f) == a df for rational a */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        fmpz_t num, den;
        mpq_t x;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        num = fmpz_init(bits / FLINT_BITS + 2);
        den = fmpz_init(bits / FLINT_BITS + 2);
        mpq_init(x);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        fmpz_randtest(&num, R, rand() % bits);
        fmpz_randtest_not_zero(&den, R, rand() % bits);
        fmpz_to_mpz(mpq_numref(x), num);
        fmpz_to_mpz(mpq_denref(x), den);
        mpq_canonicalize(x);
        
        fmpq_poly_scalar_mul_mpq(g, f, x);
        fmpq_poly_derivative(g, g);
        fmpq_poly_derivative(f, f);
        fmpq_poly_scalar_mul_mpq(f, f, x);
        
        result = fmpq_poly_equal(g, f);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(g);
            strg = fmpq_poly_to_string(f);
            printf("FAIL\n");
            printf("  lhs = %s\n", strf);
            printf("  rhs = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpz_clear(num);
        fmpz_clear(den);
        mpq_clear(x);
    }

    /* d(f g) == df g + f dg */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t lhs, rhs, f, g;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(lhs);
        fmpq_poly_init(rhs);
        
        fmpq_poly_randtest(f, R, len, rand() % bits);
        fmpq_poly_randtest(g, R, len, rand() % bits);
        
        fmpq_poly_mul(lhs, f, g);
        fmpq_poly_derivative(lhs, lhs);
        fmpq_poly_derivative(rhs, f);
        fmpq_poly_mul(rhs, rhs, g);
        fmpq_poly_derivative(g, g);
        fmpq_poly_mul(f, f, g);
        fmpq_poly_add(rhs, rhs, f);
        
        result = fmpq_poly_equal(lhs, rhs);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(lhs);
            str2 = fmpq_poly_to_string(rhs);
            printf("FAIL\n");
            printf("  lhs = %s\n", str1);
            printf("  rhs = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(lhs);
        fmpq_poly_clear(rhs);
    }
    
    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

