///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 5000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("GCD..."); fflush(stdout);

    /** Generic case, where f and g are coprime ******************************/
    
    /* Aliasing f and h in h = GCD(f, g) */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, h;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(h);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        
        fmpq_poly_gcd(h, f, g);
        fmpq_poly_gcd(f, f, g);
        
        result = fmpq_poly_equal(f, h);
        if (!result)
        {
            char * strf, * strh;
            strf = fmpq_poly_to_string(f);
            strh = fmpq_poly_to_string(h);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  h = %s\n", strh);
            free(strf);
            free(strh);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(h);
    }

    /* Aliasing g and h in h = GCD(f, g) */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, h;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(h);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        
        fmpq_poly_gcd(h, f, g);
        fmpq_poly_gcd(g, f, g);
        
        result = fmpq_poly_equal(g, h);
        if (!result)
        {
            char * strg, * strh;
            strg = fmpq_poly_to_string(g);
            strh = fmpq_poly_to_string(h);
            printf("FAIL\n");
            printf("  g = %s\n", strg);
            printf("  h = %s\n", strh);
            free(strg);
            free(strh);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(h);
    }

    /* Aliasing f and g in h = GCD(f, g) */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, h;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(h);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        
        fmpq_poly_gcd(h, f, f);
        fmpq_poly_gcd(f, f, f);
        
        result = fmpq_poly_equal(f, h);
        if (!result)
        {
            char * strf, * strh;
            strf = fmpq_poly_to_string(f);
            strh = fmpq_poly_to_string(h);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  h = %s\n", strh);
            free(strf);
            free(strh);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(h);
    }
    
    /* Verify commutativity and that h is monic */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, h;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(h);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        
        fmpq_poly_gcd(h, f, g);
        fmpq_poly_gcd(f, g, f);
        
        result = fmpq_poly_equal(f, h) && (fmpq_poly_is_zero(h) || fmpq_poly_is_monic(h));
        if (!result)
        {
            char * strf, * strh;
            strf = fmpq_poly_to_string(f);
            strh = fmpq_poly_to_string(h);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  h = %s\n", strh);
            free(strf);
            free(strh);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(h);
    }

    /* Verify GCD(f, g) | f, g */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, h, r1, r2;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(h);
        fmpq_poly_init(r1);
        fmpq_poly_init(r2);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        
        fmpq_poly_gcd(h, f, g);
        if (!fmpq_poly_is_zero(h))
        {
            fmpq_poly_mod(r1, f, h);
            fmpq_poly_mod(r2, g, h);
        }
        
        result = fmpq_poly_is_zero(r1) && fmpq_poly_is_zero(r2);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(r1);
            str2 = fmpq_poly_to_string(r2);
            printf("FAIL\n");
            printf("  r1 = %s\n", str1);
            printf("  r2 = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(h);
        fmpq_poly_clear(r1);
        fmpq_poly_clear(r2);
    }

    /** Special case, where f and g are not coprime **************************/
    
    /* Verify commutativity and that h is monic */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t a, f, g, h;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(a);
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(h);
        
        fmpq_poly_randtest(a, R, rand() % (len / 2), rand() % (bits / 2));
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        fmpq_poly_mul(f, a, f);
        fmpq_poly_mul(g, a, g);
        
        fmpq_poly_gcd(h, f, g);
        fmpq_poly_gcd(f, g, f);
        
        result = fmpq_poly_equal(f, h) && (fmpq_poly_is_zero(h) || fmpq_poly_is_monic(h));
        if (!result)
        {
            char * strf, * strh;
            strf = fmpq_poly_to_string(f);
            strh = fmpq_poly_to_string(h);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  h = %s\n", strh);
            free(strf);
            free(strh);
            abort();
        }

        fmpq_poly_clear(a);        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(h);
    }

    /* Verify GCD(f, g) | f, g */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t a, f, g, h, r1, r2;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(a);
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(h);
        fmpq_poly_init(r1);
        fmpq_poly_init(r2);
        
        fmpq_poly_randtest(a, R, rand() % (len / 2), rand() % (bits / 2));
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        fmpq_poly_mul(f, a, f);
        fmpq_poly_mul(g, a, g);
        
        fmpq_poly_gcd(h, f, g);
        if (!fmpq_poly_is_zero(h))
        {
            fmpq_poly_mod(r1, f, h);
            fmpq_poly_mod(r2, g, h);
        }
        
        result = fmpq_poly_is_zero(r1) && fmpq_poly_is_zero(r2);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(r1);
            str2 = fmpq_poly_to_string(r2);
            printf("FAIL\n");
            printf("  r1 = %s\n", str1);
            printf("  r2 = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(a);
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(h);
        fmpq_poly_clear(r1);
        fmpq_poly_clear(r2);
    }
    
    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

