///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("MOD..."); fflush(stdout);
    
    /* Aliasing r, f in r = f % g */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, q, r;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(r);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest_not_zero(g, R, rand() % len, rand() % bits);
        
        fmpq_poly_mod(r, f, g);
        fmpq_poly_mod(f, f, g);
        
        result = fmpq_poly_equal(f, r);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(f);
            str2 = fmpq_poly_to_string(r);
            printf("FAIL\n");
            printf("  f = %s\n", str1);
            printf("  r = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(r);
    }
    
    /* Aliasing r, g in r = f % g */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, q, r;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(r);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest_not_zero(g, R, rand() % len, rand() % bits);
        
        fmpq_poly_mod(r, f, g);
        fmpq_poly_mod(g,f, g);
        
        result = fmpq_poly_equal(g, r);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(g);
            str2 = fmpq_poly_to_string(r);
            printf("FAIL\n");
            printf("  g = %s\n", str1);
            printf("  r = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(r);
    }

    /* Check consistency with divrem */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, q, r1, r2;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(q);
        fmpq_poly_init(r1);
        fmpq_poly_init(r2);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest_not_zero(g, R, rand() % len, rand() % bits);
        
        fmpq_poly_mod(r1, f, g);
        fmpq_poly_divrem(q, r2, f, g);
        
        result = fmpq_poly_equal(r1, r2);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(r1);
            str2 = fmpq_poly_to_string(r2);
            printf("FAIL\n");
            printf("  r1 = %s\n", str1);
            printf("  r2 = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(q);
        fmpq_poly_clear(r1);
        fmpq_poly_clear(r2);
    }
    
    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

