///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("MUL..."); fflush(stdout);
    
    /* Aliasing f and h in h = f * g */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, h;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(h);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        
        fmpq_poly_mul(h, f, g);
        fmpq_poly_mul(f, f, g);
        
        result = fmpq_poly_equal(f, h);
        if (!result)
        {
            char * strf, * strh;
            strf = fmpq_poly_to_string(f);
            strh = fmpq_poly_to_string(h);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  h = %s\n", strh);
            free(strf);
            free(strh);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(h);
    }
    
    /* Aliasing g and h in h = f * g */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, h;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(h);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        
        fmpq_poly_mul(h, f, g);
        fmpq_poly_mul(g, f, g);
        
        result = fmpq_poly_equal(g, h);
        if (!result)
        {
            char * strg, * strh;
            strg = fmpq_poly_to_string(g);
            strh = fmpq_poly_to_string(h);
            printf("FAIL\n");
            printf("  g = %s\n", strg);
            printf("  h = %s\n", strh);
            free(strg);
            free(strh);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(h);
    }

    /* Aliasing f and g in h = f * g */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, h;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(h);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        
        fmpq_poly_mul(h, f, f);
        fmpq_poly_mul(f, f, f);
        
        result = fmpq_poly_equal(f, h);
        if (!result)
        {
            char * strf, * strh;
            strf = fmpq_poly_to_string(f);
            strh = fmpq_poly_to_string(h);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  h = %s\n", strh);
            free(strf);
            free(strh);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(h);
    }
    
    /* Verify commutativity */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, h;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(h);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        
        fmpq_poly_mul(h, f, g);
        fmpq_poly_mul(f, g, f);
        
        result = fmpq_poly_equal(f, h);
        if (!result)
        {
            char * strf, * strh;
            strf = fmpq_poly_to_string(f);
            strh = fmpq_poly_to_string(h);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  h = %s\n", strh);
            free(strf);
            free(strh);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(h);
    }

    /* Verify (f + g) * h = f * h + g * h */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, h, lhs, rhs;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(h);
        fmpq_poly_init(lhs);
        fmpq_poly_init(rhs);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(h, R, rand() % len, rand() % bits);
        
        fmpq_poly_add(lhs, f, g);
        fmpq_poly_mul(lhs, lhs, h);
        
        fmpq_poly_mul(f, f, h);
        fmpq_poly_mul(g, g, h);
        fmpq_poly_add(rhs, f, g);
        
        result = fmpq_poly_equal(lhs, rhs);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(lhs);
            str2 = fmpq_poly_to_string(rhs);
            printf("FAIL\n");
            printf("  lhs = %s\n", str1);
            printf("  rhs = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(h);
        fmpq_poly_clear(lhs);
        fmpq_poly_clear(rhs);
    }
    
    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

