///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("REVERSE..."); fflush(stdout);
    
    /* Check aliasing */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        ulong len = 100, bits = 100;
        ulong n;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        n = rand() % (2 * fmpq_poly_length(f) + 1);
        
        fmpq_poly_reverse(g, f, n);
        fmpq_poly_reverse(f, f, n);
        
        result = fmpq_poly_equal(f, g);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(f);
            strg = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  F = "); fmpz_poly_print(f->num); printf(".."); fmpz_print(f->den); printf("\n");
            printf("  G = "); fmpz_poly_print(g->num); printf(".."); fmpz_print(g->den); printf("\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            printf("  n = %lu\n", n);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
    }

    /* Check that if n == len(f) the operation is a self-inverse */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        
        fmpq_poly_reverse(g, f, fmpq_poly_length(f));
        fmpq_poly_reverse(g, g, fmpq_poly_length(f));
        
        result = fmpq_poly_equal(f, g);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(f);
            strg = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
    }

    /* Check that rev(f + g, n) == rev(f, n) + rev(g, n) */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, lhs, rhs;
        ulong n;
        ulong len = 100, bits = 100;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(lhs);
        fmpq_poly_init(rhs);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        n = rand() % (3 * len / 2);
        
        fmpq_poly_reverse(lhs, f, n);
        fmpq_poly_reverse(rhs, g, n);
        fmpq_poly_add(rhs, lhs, rhs);
        fmpq_poly_add(lhs, f, g);
        fmpq_poly_reverse(lhs, lhs, n);
        
        result = fmpq_poly_equal(lhs, rhs);
        if (!result)
        {
            char * strf, * strg, * strlhs, * strrhs;
            strf = fmpq_poly_to_string(f);
            strg = fmpq_poly_to_string(g);
            strlhs = fmpq_poly_to_string(lhs);
            strrhs = fmpq_poly_to_string(rhs);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            printf("  lhs = %s\n", strlhs);
            printf("  rhs = %s\n", strrhs);
            free(strf);
            free(strg);
            free(strlhs);
            free(strrhs);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(lhs);
        fmpq_poly_clear(rhs);
    }

    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

