///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

#include <math.h>

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("SCALAR_MUL_MPQ..."); fflush(stdout);
    
    /* Aliasing f and g in g = c * f */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        ulong len = 100, bits = 100;
        fmpz_t cnum, cden;
        mpq_t c;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        cnum = fmpz_init(bits / FLINT_BITS + 2);
        cden = fmpz_init(bits / FLINT_BITS + 2);
        mpq_init(c);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpz_randtest(&cnum, R, rand() % bits);
        fmpz_randtest_not_zero(&cden, R, rand() % bits);
        fmpz_to_mpz(mpq_numref(c), cnum);
        fmpz_to_mpz(mpq_denref(c), cden);
        mpq_canonicalize(c);
        
        fmpq_poly_scalar_mul_mpq(g, f, c);
        fmpq_poly_scalar_mul_mpq(f, f, c);
        
        result = fmpq_poly_equal(f, g);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(f);
            strg = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpz_clear(cnum);
        fmpz_clear(cden);
        mpq_clear(c);
    }
    
    /* c (f + g) == c f + c g */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, lhs, rhs;
        ulong len = 100, bits = 100;
        fmpz_t cnum, cden;
        mpq_t c;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(lhs);
        fmpq_poly_init(rhs);
        cnum = fmpz_init(bits / FLINT_BITS + 2);
        cden = fmpz_init(bits / FLINT_BITS + 2);
        mpq_init(c);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        fmpz_randtest(&cnum, R, rand() % bits);
        fmpz_randtest_not_zero(&cden, R, rand() % bits);
        fmpz_to_mpz(mpq_numref(c), cnum);
        fmpz_to_mpz(mpq_denref(c), cden);
        mpq_canonicalize(c);
        
        fmpq_poly_add(lhs, f, g);
        fmpq_poly_scalar_mul_mpq(lhs, lhs, c);

        fmpq_poly_scalar_mul_mpq(f, f, c);
        fmpq_poly_scalar_mul_mpq(g, g, c);
        fmpq_poly_add(rhs, f, g);
        
        result = fmpq_poly_equal(lhs, rhs);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(lhs);
            str2 = fmpq_poly_to_string(rhs);
            printf("FAIL\n");
            printf("  lhs = %s\n", str1);
            printf("  rhs = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(lhs);
        fmpq_poly_clear(rhs);
        fmpz_clear(cnum);
        fmpz_clear(cden);
        mpq_clear(c);
    }

    /* d (c f) == (c d) f */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, lhs, rhs;
        ulong len = 100, bits = 100;
        fmpz_t cnum, cden, dnum, dden;
        mpq_t c, d, x;
        
        fmpq_poly_init(f);
        fmpq_poly_init(lhs);
        fmpq_poly_init(rhs);
        cnum = fmpz_init(bits / FLINT_BITS + 2);
        cden = fmpz_init(bits / FLINT_BITS + 2);
        dnum = fmpz_init(bits / FLINT_BITS + 2);
        dden = fmpz_init(bits / FLINT_BITS + 2);
        mpq_init(c);
        mpq_init(d);
        mpq_init(x);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpz_randtest(&cnum, R, rand() % bits);
        fmpz_randtest_not_zero(&cden, R, rand() % bits);
        fmpz_randtest(&dnum, R, rand() % bits);
        fmpz_randtest_not_zero(&dden, R, rand() % bits);
        fmpz_to_mpz(mpq_numref(c), cnum);
        fmpz_to_mpz(mpq_denref(c), cden);
        fmpz_to_mpz(mpq_numref(d), dnum);
        fmpz_to_mpz(mpq_denref(d), dden);
        mpq_canonicalize(c);
        mpq_canonicalize(d);
        
        fmpq_poly_scalar_mul_mpq(lhs, f, c);
        fmpq_poly_scalar_mul_mpq(lhs, lhs, d);

        mpq_mul(x, c, d);
        fmpq_poly_scalar_mul_mpq(rhs, f, x);
        
        result = fmpq_poly_equal(lhs, rhs);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(lhs);
            str2 = fmpq_poly_to_string(rhs);
            printf("FAIL\n");
            printf("  lhs = %s\n", str1);
            printf("  rhs = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(lhs);
        fmpq_poly_clear(rhs);
        fmpz_clear(cnum);
        fmpz_clear(cden);
        fmpz_clear(dnum);
        fmpz_clear(dden);
        mpq_clear(c);
        mpq_clear(d);
        mpq_clear(x);
    }

    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

