///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

#include <math.h>

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("SCALAR_MUL_MPZ..."); fflush(stdout);
    
    /* Aliasing f and g in g = c * f */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        ulong len = 100, bits = 100;
        fmpz_t c;
        mpz_t mc;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        c = fmpz_init(bits / FLINT_BITS + 2);
        mpz_init(mc);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpz_randtest(&c, R, rand() % bits);
        fmpz_to_mpz(mc, c);
        
        fmpq_poly_scalar_mul_mpz(g, f, mc);
        fmpq_poly_scalar_mul_mpz(f, f, mc);
        
        result = fmpq_poly_equal(f, g);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(f);
            strg = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpz_clear(c);
        mpz_clear(mc);
    }
    
    /* c (f + g) == c f + c g */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, lhs, rhs;
        ulong len = 100, bits = 100;
        fmpz_t c;
        mpz_t mc;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(lhs);
        fmpq_poly_init(rhs);
        c = fmpz_init(bits / FLINT_BITS + 2);
        mpz_init(mc);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        fmpz_randtest(&c, R, rand() % bits);
        fmpz_to_mpz(mc, c);
        
        fmpq_poly_add(lhs, f, g);
        fmpq_poly_scalar_mul_mpz(lhs, lhs, mc);

        fmpq_poly_scalar_mul_mpz(f, f, mc);
        fmpq_poly_scalar_mul_mpz(g, g, mc);
        fmpq_poly_add(rhs, f, g);
        
        result = fmpq_poly_equal(lhs, rhs);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(lhs);
            str2 = fmpq_poly_to_string(rhs);
            printf("FAIL\n");
            printf("  lhs = %s\n", str1);
            printf("  rhs = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(lhs);
        fmpq_poly_clear(rhs);
        fmpz_clear(c);
        mpz_clear(mc);
    }

    /* d (c f) == (c d) f */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, lhs, rhs;
        ulong len = 100, bits = 100;
        fmpz_t c, d;
        mpz_t mc, md, mx;
        
        fmpq_poly_init(f);
        fmpq_poly_init(lhs);
        fmpq_poly_init(rhs);
        c = fmpz_init(bits / FLINT_BITS + 2);
        d = fmpz_init(bits / FLINT_BITS + 2);
        mpz_init(mc);
        mpz_init(md);
        mpz_init(mx);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpz_randtest(&c, R, rand() % bits);
        fmpz_randtest(&d, R, rand() % bits);
        fmpz_to_mpz(mc, c);
        fmpz_to_mpz(md, d);
        
        fmpq_poly_scalar_mul_mpz(lhs, f, mc);
        fmpq_poly_scalar_mul_mpz(lhs, lhs, md);

        mpz_mul(mx, mc, md);
        fmpq_poly_scalar_mul_mpz(rhs, f, mx);
        
        result = fmpq_poly_equal(lhs, rhs);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(lhs);
            str2 = fmpq_poly_to_string(rhs);
            printf("FAIL\n");
            printf("  lhs = %s\n", str1);
            printf("  rhs = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(lhs);
        fmpq_poly_clear(rhs);
        fmpz_clear(c);
        fmpz_clear(d);
        mpz_clear(mc);
        mpz_clear(md);
        mpz_clear(mx);
    }

    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

