///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

#include <math.h>

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 1000;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("SCALAR_MUL_SI..."); fflush(stdout);
    
    /* Aliasing f and g in g = c * f */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g;
        ulong len = 100, bits = 100;
        long c;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        c = rand() % LONG_MAX;
        if (rand() % 2)
            c = -c;
        
        fmpq_poly_scalar_mul_si(g, f, c);
        fmpq_poly_scalar_mul_si(f, f, c);
        
        result = fmpq_poly_equal(f, g);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpq_poly_to_string(f);
            strg = fmpq_poly_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
    }
    
    /* c (f + g) == c f + c g */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, g, lhs, rhs;
        ulong len = 100, bits = 100;
        long c;
        
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        fmpq_poly_init(lhs);
        fmpq_poly_init(rhs);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        c = rand() % LONG_MAX;
        if (rand() % 2)
            c = -c;
        
        fmpq_poly_add(lhs, f, g);
        fmpq_poly_scalar_mul_si(lhs, lhs, c);

        fmpq_poly_scalar_mul_si(f, f, c);
        fmpq_poly_scalar_mul_si(g, g, c);
        fmpq_poly_add(rhs, f, g);
        
        result = fmpq_poly_equal(lhs, rhs);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(lhs);
            str2 = fmpq_poly_to_string(rhs);
            printf("FAIL\n");
            printf("  lhs = %s\n", str1);
            printf("  rhs = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
        fmpq_poly_clear(lhs);
        fmpq_poly_clear(rhs);
    }

    /* d (c f) == (c d) f */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t f, lhs, rhs;
        ulong len = 100, bits = 100;
        long c, d;
        
        fmpq_poly_init(f);
        fmpq_poly_init(lhs);
        fmpq_poly_init(rhs);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        c = rand() % ((long) sqrt((double) LONG_MAX));
        if (rand() % 2)
            c = -c;
        d = rand() % ((long) sqrt((double) LONG_MAX));
        if (rand() % 2)
            d = -d;
        
        fmpq_poly_scalar_mul_si(lhs, f, c);
        fmpq_poly_scalar_mul_si(lhs, lhs, d);

        fmpq_poly_scalar_mul_si(rhs, f, c * d);
        
        result = fmpq_poly_equal(lhs, rhs);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(lhs);
            str2 = fmpq_poly_to_string(rhs);
            printf("FAIL\n");
            printf("  lhs = %s\n", str1);
            printf("  rhs = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(f);
        fmpq_poly_clear(lhs);
        fmpq_poly_clear(rhs);
    }

    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

