///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpq_poly-testr.h"

int main(int argc, char *argv[])
{
    gmp_randstate_t R;
    ulong i, N = 300;
    
    gmp_randinit_default(R);
    gmp_randseed_ui(R, SEED_GMP);
    srand(SEED_SYS);
    
    printf("XGCD..."); fflush(stdout);

    /** Generic case, where f and g are coprime ******************************/
    
    /* Verify d == s f + t g */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t d, s, t, f, g;
        ulong len = 70, bits = 70;
        
        fmpq_poly_init(d);
        fmpq_poly_init(s);
        fmpq_poly_init(t);
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        
        fmpq_poly_xgcd(d, s, t, f, g);
        fmpq_poly_mul(f, s, f);
        fmpq_poly_mul(g, t, g);
        fmpq_poly_add(f, f, g);
         
        result = fmpq_poly_equal(d, f);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(d);
            str2 = fmpq_poly_to_string(f);
            printf("FAIL\n");
            printf("  lhs = %s\n", str1);
            printf("  rhs = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(d);
        fmpq_poly_clear(s);
        fmpq_poly_clear(t);
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
    }

    /* Verify consistency with gcd */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t d, s, t, f, g;
        ulong len = 70, bits = 70;
        
        fmpq_poly_init(d);
        fmpq_poly_init(s);
        fmpq_poly_init(t);
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        
        fmpq_poly_xgcd(d, s, t, f, g);
        fmpq_poly_gcd(f, f, g);
        
        result = fmpq_poly_equal(d, f);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(d);
            str2 = fmpq_poly_to_string(f);
            printf("FAIL\n");
            printf("  d = %s\n", str1);
            printf("  f = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(d);
        fmpq_poly_clear(s);
        fmpq_poly_clear(t);
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
    }

    /** Special case, where f and g are not coprime **************************/
    
    /* Verify d == s f + t g */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t a, d, s, t, f, g;
        ulong len = 50, bits = 70;
        
        fmpq_poly_init(a);
        fmpq_poly_init(d);
        fmpq_poly_init(s);
        fmpq_poly_init(t);
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        
        fmpq_poly_randtest(a, R, rand() % (len / 2), rand() % (bits / 2));
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        fmpq_poly_mul(f, a, f);
        fmpq_poly_mul(g, a, g);
        
        fmpq_poly_xgcd(d, s, t, f, g);
        fmpq_poly_mul(f, s, f);
        fmpq_poly_mul(g, t, g);
        fmpq_poly_add(f, f, g);
         
        result = fmpq_poly_equal(d, f);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(d);
            str2 = fmpq_poly_to_string(f);
            printf("FAIL\n");
            printf("  lhs = %s\n", str1);
            printf("  rhs = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(a);
        fmpq_poly_clear(d);
        fmpq_poly_clear(s);
        fmpq_poly_clear(t);
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
    }

    /* Verify consistency with gcd */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpq_poly_t a, d, s, t, f, g;
        ulong len = 50, bits = 70;
        
        fmpq_poly_init(a);
        fmpq_poly_init(d);
        fmpq_poly_init(s);
        fmpq_poly_init(t);
        fmpq_poly_init(f);
        fmpq_poly_init(g);
        
        fmpq_poly_randtest(a, R, rand() % (len / 2), rand() % (bits / 2));
        fmpq_poly_randtest(f, R, rand() % len, rand() % bits);
        fmpq_poly_randtest(g, R, rand() % len, rand() % bits);
        fmpq_poly_mul(f, a, f);
        fmpq_poly_mul(g, a, g);
        
        fmpq_poly_xgcd(d, s, t, f, g);
        fmpq_poly_gcd(f, f, g);
        
        result = fmpq_poly_equal(d, f);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpq_poly_to_string(d);
            str2 = fmpq_poly_to_string(f);
            printf("FAIL\n");
            printf("  d = %s\n", str1);
            printf("  f = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpq_poly_clear(a);
        fmpq_poly_clear(d);
        fmpq_poly_clear(s);
        fmpq_poly_clear(t);
        fmpq_poly_clear(f);
        fmpq_poly_clear(g);
    }
    
    printf("PASS\n");
    
    gmp_randclear(R);
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

